;Sol.asm

;(c) Michael Pearce, 1997-98 and now 1999

#include "usgard.h"            

#include "solskey.h"



LEVELNUM        = TEXT_MEM      ;1 BYTE for the current level
LEVEL           = TEXT_MEM+1    ;112 BYTES Current Level Map
SOLSTRUCT       = TEXT_MEM+113  ;9 BYTES
                                ;   BYTE character number
                                ;   BYTE x position
                                ;   BYTE y position
                                ;   BYTE delay for animations
                                ;   BYTE Offset from beginning of sprite anim
                                ;   BYTE for flags
                                ;   BYTE for extra data
                                ;   WORD for address of animation
REFRESHCOUNT    = TEXT_MEM+122  ;1 BYTE for the screen refresh count
TEMP            = TEXT_MEM+123  ;1 BYTE 
GAMEFLAGS       = TEXT_MEM+124  ;1 BYTE for flags for the game
NUMSCROLL       = TEXT_MEM+125  ;1 BYTE the current size of fireball scroll
NUMFIREBALLS    = TEXT_MEM+126  ;1 BYTE number of fireballs in scroll
NUMLEVELS       = TEXT_MEM+127  ;1 BYTE number of levels in the chosen world
LEVELPTR        = TEXT_MEM+128  ;1 WORD addr of current lvl's compressed data
JUMPOFFSET      = TEXT_MEM+130  ;1 BYTE for the position in the jump
LASTKEYS        = TEXT_MEM+131  ;1 BYTE to store the last keypress
LASTFLAGS       = TEXT_MEM+132  ;1 BYTE for sol's last flags (in structure)
BLOCKXY         = TEXT_MEM+133  ;1 WORD for the mapped x,y of block to break
NUMENEMIES      = TEXT_MEM+135  ;1 BYTE for the number of enemies
WANDXY          = TEXT_MEM+137  ;1 WORD for the mapped x,y of wand block
LASTWANDTRY     = TEXT_MEM+139  ;1 WORD mapped x,y of last attemped wand
VIRTUALSCREEN   = TEXT_MEM+141  ;1 WORD for the location of virtual screen
INTTIMER        = TEXT_MEM+143  ;1 BYTE timer so game runs at constant speed
GAMETIMER       = TEXT_MEM+144  ;1 WORD time remaining to play level
SOLLIVES        = TEXT_MEM+146  ;1 BYTE number of lives sol has remaining
NUMWORLDS       = TEXT_MEM+147  ;1 BYTE number of worlds in the VAT
WORLDTEXTADDR   = TEXT_MEM+148  ;1 WORD for address of world name
NEXTLEVELPTR    = TEXT_MEM+150  ;1 WORD addr of next lvl's compressed data
TEMPWORD        = TEXT_MEM+152  ;1 WORD temp word
FBSTRUCT        = TEXT_MEM+154  ;9 BYTES struct for fireball

ENEMYSTRUCTS    = TEXT_MEM2     ;up to 18 enemies at 9 bytes each

.org 0

SolsKeyText:
.db "  Solomons Key v1.1  ",0

Start:

   call CLEARLCD

   ld hl,(VAT_END)              ;one byte before end of VAT
   ld a,h                       ;\
   sub 4                        ; |decrease hl by 1024
   ld h,a                       ;/
   ld (VIRTUALSCREEN),hl        ;save location of virtual screen
   ld de,(FIRST_FREE)           ;first free byte of memory
   or a                         ;\ is new area of virtual screen
   sbc hl,de                    ;/  all in free memory?
   jr nc,StartSolsKey           ;got enough memory so start 'er up
   ld hl,$0403                  ;else display message of not enough mem
   ld ($800C),hl
   ld hl,&MemErrorText
   call D_ZT_STR
   call OTH_PAUSE
   ret                          ;exit the game

MemErrorText:
.db "Need 1KB free!",0


AuthorText:
.db "Michael Pearce",0
EmailText:
.db "mikep@anet-dfw.com",0

StartSolsKey:
   ld hl,$0002
   ld ($800C),hl
   ld hl,&SolsKeyText
   call D_ZT_STR
   ld hl,$0305
   ld ($800C),hl
   ld hl,&AuthorText
   call D_ZT_STR
   ld hl,$0106
   ld ($800C),hl
   ld hl,&EmailText
   call D_ZT_STR
   call OTH_PAUSE
   call CLEARLCD

   ld a,4
   out (5),a

   ld a,3                       
   ld (SOLLIVES),a              ;start new game with 3 Sols

   xor a
   ld (GAMEFLAGS),a             ;initial game flags are all reset
   ld (LEVELNUM),a              ;initial (1st) level is 0

   inc a
   ld (NUMSCROLL),a             ;start new game with scroll length == 2


   call &ChooseWorld            ;hl has addr of level name
   ret c                        ;no levels in VAT so end game

   ld (WORLDTEXTADDR),hl        ;save the address of the text
   call &DispLevelScreen        ;returns addr of 1st level in hl

   ld (LEVELPTR),hl             ;save the address of the current level

   call &LoadLevel
   call &InitLevel

   ld hl,&TimerInterrupt
   call INT_INSTALL             ;install the timer interrupt
   ret c                        ;exit the game if could not install the interrupt

MainGameLoop:
   ld hl,GAMEFLAGS
   ld a,%00111111
   out (1),a
   in a,(1)
   bit 6,a                      ;exit pressed?
   jr z,GameOver                ;yes, so game is over
   bit 7,a                      ;more pressed?
   call z,&PauseGame            ;yes, so pause game
   bit ISFINISHED,(hl)          ;change to load next level
   call nz,&LoadNextLevel
   bit ISSOLDEAD,(hl)
   call nz,&SolHasDied
   bit ISGAMEOVER,(hl)
   jr nz,GameOver
   bit WONGAME,(hl)
   jr nz,GameWon
   call &MoveSol
   call &MoveFireball
   call &MoveEnemies
   call &DrawSprites
   jr MainGameLoop

GameWon:
   call &Transition
   ld hl,$0203
   ld ($800C),hl
   ld hl,&WinnerText
   set 3,(iy+5)                 ;reverse video writing
   call D_ZT_STR
   res 3,(iy+5)
   call OTH_PAUSE
GameOver:
   call INT_REMOVE              ;remove the timer interrupt
   ret                          ;return to shell

WinnerText:
.db "All levels beaten!",0



;input:
;  hl - addr of GAMEFLAGS
;destroys:
;  af, b
PauseGame:
   bit CHEATENABLED,(hl)
   ret z                        ;cheat not enabled
   set ISFINISHED,(hl)          ;go to next level
   ret



;input:
;  none
;destroys:
;  af, bc, de
Transition:
   push hl
   ld b,8
   ld c,1
T_LoopOuter:
   ld hl,VIDEO_MEM
   ld de,1024
   rrc c
   ld a,20
T_HaltLoop:
   halt
   dec a
   jr nz,T_HaltLoop
T_LoopInner:
   ld a,c
   or (hl)
   ld (hl),a
   inc hl
   dec de
   ld a,d
   or e
   jr nz,T_LoopInner
   djnz T_LoopOuter
   pop hl
   ret



;input:
;  hl has addr of GAMEFLAGS
;destroys:
;  af,bc,de
SolHasDied:
   ld b,100
SHD_Pause:
   halt
   djnz SHD_Pause
   res ISSOLDEAD,(hl)
   push hl
   ld hl,SOLLIVES
   dec (hl)                     ;set the carry flag if less than 0 lives left
   ld a,(hl)
   or a
   jr z,SolDiedAndGameOver
   call &DispLevelScreen
   call &LoadLevel
   call &InitLevel
   pop hl
   res GOTKEY,(hl)              ;sol no longer has the key
   ret
SolDiedAndGameOver:
   pop hl
   set ISGAMEOVER,(hl)
   ret


;input:
;  hl has addr of GAMEFLAGS
;don't destroy hl!
LoadNextLevel:
   call &Transition
   res GOTKEY,(hl)              ;does not have key for this new level
   set WONGAME,(hl)             ;assume this is last level
   ld a,(NUMLEVELS)
   ex de,hl                     ;put GAMEFLAGS in de
   ld hl,LEVELNUM
   inc (hl)                     ;now at next level
   cp (hl)
   ex de,hl                     ;put GAMEFLAGS in hl
   ret z                        ;finished last level so return with WONGAME set
   res WONGAME,(hl)             ;didn't finish last level
   res ISFINISHED,(hl)          ;reset for new level
   push hl
   call &DispLevelScreen

   ld hl,(NEXTLEVELPTR)
   ld (LEVELPTR),hl             ;next level is now the current level
   call &LoadLevel
   call &InitLevel
   pop hl                       ;restore GAMEFLAGS
   ret



DrawSprites:
   ld hl,REFRESHCOUNT
   dec (hl)
   ret nz                       ;only draw sprites when REFRESHCOUNT down to 0
   ld (hl),REFRESHNUM           ;(REFRESHNUM is how often to refresh screen)

   ld bc,1024
   ld hl,GRAPH_MEM
   ld de,(VIRTUALSCREEN)
   ldir                         ;copy the background tiles to virtual screen

   ;draw fireball
   ld ix,FBSTRUCT
   ld a,(ix+EXTRA)
   or a
   jr z,DontDrawFB              ;no fireball so nothing to draw
   ld b,(ix+XPOS)
   ld c,(ix+YPOS)
   ld h,(ix+SPRTADDR)
   ld l,(ix+SPRTADDR+1)
   call &PutSpriteOR
DontDrawFB:

   ;draw sol
   ld ix,SOLSTRUCT
   ld b,(ix+XPOS)
   ld c,(ix+YPOS)
   ld h,(ix+SPRTADDR)
   ld l,(ix+SPRTADDR+1)
   call &PutSpriteOR

   ;draw enemies
   ; (b,c) has position of sol for collision test
   ld d,b
   ld e,c
   ld ix,ENEMYSTRUCTS
   ld a,(NUMENEMIES)
DrawEnemiesLoop:
   or a
   jr z,FinishDrawing           ;drawn all enemies
   push af
   ld b,(ix+XPOS)
   ld c,(ix+YPOS)
   ld h,(ix+SPRTADDR)
   ld l,(ix+SPRTADDR+1)
   call &PutSpriteOR
   ld a,5                       ;size of sprite (used smaller to give a
                                ; extra movement around enemies)
   call &SpriteCollision        ;a=size, b,c=sprite1(enemy), d,e=sprite2(sol)
   jr c,NoEnemyCollision        ;sol didn't run into an enemy
   pop af                       ;restore stack
   jr SolDied
NoEnemyCollision:
   ;check to see if collided with spitter also
   ;which could have its "normal" mapped x,y position in EXTRA byte
   ld a,(ix+CHARNUM)
   cp FIRSTSPITTER                ;is CHARNUM less than FIRSTSPITTER?
   jr c,NoEnemyCollision2         ;nope, not spitter so jump
   ld a,LASTSPITTER
   cp (ix+CHARNUM)                ;is CHARNUM greater than LASTSPITTER?
   jr c,NoEnemyCollision2         ;nope, not spitter so jump
   push de
   call &GetPositionFromExtra   ;b,c has enemy spitter x,y position
   pop de                       ;d,e has sol x,y position
   ld a,5                       ;size of sprite (used smaller to give a
                                ; extra movement around enemies)
   call &SpriteCollision        ;check if there is a collision
   jr c,NoEnemyCollision2       ;no collision with Spitters either
   pop af
   jr SolDied                   ;else collision so sol died
NoEnemyCollision2:

   ld bc,SIZEOFSTRUCT
   add ix,bc
   pop af
   dec a
   jr DrawEnemiesLoop

   ;this section of code makes sure that the timing of the game is constant
FinishDrawing:
   halt                         ;wait for an interrupt to occur
   ld a,(INTTIMER)
   cp GAMESPEED                 ;is the timer greater than GAMESPEED?
   jr c,FinishDrawing           ;not greater so keep halting
   xor a
   ld (INTTIMER),a              ;reset the timer

   ;now redraw the screen
   ld bc,1024
   ld hl,(VIRTUALSCREEN)
   ld de,VIDEO_MEM
   ldir                         ;copy the virtual screen to video mem

   ;now decrement the game timer
   ld hl,$3920
   ld ($8333),hl
   ld hl,&TimeText
   call D_ZM_STR
   ld hl,(GAMETIMER)
   ld de,4                      ;value that timer decrements
   or a                         ;reset carry for sbc
   sbc hl,de                    ;decrement timer
   ld (GAMETIMER),hl            ;save new time
   push af
   call DM_HL_DECI
   pop af
   ret nz                       ;time didn't run out so return
SolDied:
   ld hl,GAMEFLAGS
   set ISSOLDEAD,(hl)           ;set flag saying sol just died
   ret

TimeText:
.db "Time ",0


;destroys:
;  af,bc,de,hl
DrawScroll:
   ld bc,$4838
   ld hl,&ScrollStart
   call &AllignedSprite         ;draw on tile virtual screen
   ld a,(NUMSCROLL)
   ld d,a
   ld a,(NUMFIREBALLS)
   ld e,a
DSFireballs:
   ld a,8                       ;\
   add a,b                      ; | move sprite over 8 pixels
   ld b,a                       ;/
   dec d
   bit 7,d                      ;is number negative?
   jr nz,DSFinishedFireballs    ;yes, so finished drawing scroll
   ld hl,&ScrollFull            ;assume drawing scroll w/ fireball
   dec e
   bit 7,e                      
   jr z,DrawMidScroll           ;yes, there is a fireball so draw
   ld hl,&ScrollEmpty           ;wrong assumption, so draw empty
DrawMidScroll:
   call &AllignedSprite         ;draw on tile virtual screen
   jr DSFireballs
DSFinishedFireballs:
   ld hl,&ScrollEnd
   call &AllignedSprite
   ret


;Input:
;  none
;Output:
;  updates each enemies' structure
;Destroys:
;  all
MoveEnemies:
   ld ix,ENEMYSTRUCTS
   ld a,(NUMENEMIES)
MELoop:
   or a                         ;are there zero enemies left to move?
   ret z                        ;yes, so return
   push af
   ld b,(ix+XPOS)
   ld c,(ix+YPOS)
   ld a,(ix+CHARNUM)
   call &MoveSingleEnemy
   ld (ix+XPOS),b               ;save enemy's x
   ld (ix+YPOS),c               ;save enemy's y
   ld de,SIZEOFSTRUCT
   add ix,de                    ;point to next enemy structure
   pop af                       ;restore the number of enemies
   dec a                        ;one less enemy
   jr MELoop                    ;move the next enemy
   ;return is within the loop



;move one of the enemies
;  a has char
;  bc has x,y position of enemy
;  ix has enemy's structure addr
;output:
;  bc has x,y position
;destroys:
;  a, de,hl
MoveSingleEnemy:
   dec (ix+DELAY)
   ret nz                       ;only move enemy when delay is down to 0
   ld d,a                       ;save character number
   and %00111111                ;dont look at 2 MSB for jump table
   dec a                        ;sol is first character, so forget about him
   ld e,a                       ;\
   add a,a                      ; | mult by 3 (size of jp instruction)
   add a,e                      ; | to get addr of enemy to move in jp table
   ld e,a                       ;/
   ld a,d                       ;restore character number for use in routines
   ld d,0
   ld hl,&MoveEnemiesTable
   add hl,de
   push hl                      ;push address of where to go in jump table
   ret                          ;put this address into the program counter
MoveEnemiesTable:
   jp &MoveSpark                ;$41
   jp &MoveSpark                ;$42
   jp &ChangeSpark              ;$43
   jp &ChangeSpark              ;$44
   jp &MoveFlame                ;$45
   jp &MoveSkullSetup           ;$46
   jp &MoveSkull                ;$47
   jp &MoveSpitterUp            ;$48
   jp &MoveSpitterDown          ;$49
   jp &MoveSpitterLeft          ;$4A
   jp &MoveSpitterRight         ;$4B



;Input:
;  a has character
;  b,c has the x,y coordinate
;  ix has the structure of the enemy
;Output:
;  bc has updated x,y coordinate
;  SPRTADDR updated
MoveSpitterLeft:
   ld hl,&EmptyAni
   call &SpriteAnim
   ld d,(ix+FLAGS)
   bit ISINITIALIZED,d
   jr nz,MSL_AlreadySetup
   call &SetupSpitter
   inc b \ inc b                ;\ move right so starts spitting at
                                ;/  block directly to the left of enemy
   ret
MSL_AlreadySetup:
   bit ISSPITTING,d
   jr nz,MSL_ContSpitting
MSL_NotSpitting:
   ld hl,SOLSTRUCT+YPOS         ;we only want to spit when Sol is close
   ld d,20                      ;range to the L or R where we will spit
   ld a,c                       ;\
   add a,d                      ; |
   cp (hl)                      ; | return if sol is not in range 
   ret c                        ; |  of the enemy
   ld a,(hl)                    ; |
   add a,d                      ; |
   cp c                         ; |
   ret c                        ;/
   set ISSPITTING,(ix+FLAGS)
   ret
MSL_ContSpitting:
   ld hl,&SpitterAni
   call &SpriteAnim
   dec b \ dec b                ;assume can continue moving left
   ld a,b                       ;\  must be at a position where (x pos)%8==0
                                ; | to check left, else keep going left
   and 7                        ;/ 
   call z,&CheckSparkLeft       ;hl, de set to tile addr, mapped pos
   ret nc                       ;ok to keep moving so return
   call nz,&MS_RemoveBlock      ;call the MoveSkull remove block routine
                                ; to remove a block if block was spit on
                                ;if z set, spitting at border of screen,
                                ;  so there is no block to remove
   call &GetPositionFromExtra
   inc b \ inc b                ;\ move right so starts spitting at
   res ISSPITTING,(ix+FLAGS)    ;done spitting so reset flag
   ret

MoveSpitterRight:
   ld hl,&EmptyAni
   call &SpriteAnim
   ld d,(ix+FLAGS)
   bit ISINITIALIZED,d
   jr nz,MSR_AlreadySetup
   call &SetupSpitter
MSR_AlreadySetup:
   bit ISSPITTING,d
   jr nz,MSR_ContSpitting
MSR_NotSpitting:
   ld hl,SOLSTRUCT+YPOS         ;we only want to spit when Sol is close
   ld d,20                      ;range to the L or R where we will spit
   ld a,c                       ;\
   add a,d                      ; |
   cp (hl)                      ; | return if sol is not in range 
   ret c                        ; |  of the enemy
   ld a,(hl)                    ; |
   add a,d                      ; |
   cp c                         ; |
   ret c                        ;/
   set ISSPITTING,(ix+FLAGS)
   ret
MSR_ContSpitting:
   ld hl,&SpitterAni
   call &SpriteAnim
   inc b \ inc b                ;assume can continue moving right
   ld a,b                       ;\  must be at a position where (x pos)%8==2
   sub 2                        ; | to check right, else keep going right
   and 7                        ;/ 
   call z,&CheckSparkRight      ;hl, de set to tile addr, mapped pos
   ret nc                       ;ok to keep moving so return
   call nz,&MS_RemoveBlock      ;call the MoveSkull remove block routine
                                ; to remove a block if block was spit on
                                ;if z set, spitting at border of screen,
                                ;  so there is no block to remove
   call &GetPositionFromExtra
   res ISSPITTING,(ix+FLAGS)    ;done spitting so reset flag
   ret

MoveSpitterUp:
   ld hl,&EmptyAni
   call &SpriteAnim
   ld d,(ix+FLAGS)
   bit ISINITIALIZED,d
   jr nz,MSU_AlreadySetup
   call &SetupSpitter
   inc c \ inc c                ;\ move down so starts spitting at
MSU_AlreadySetup:
   bit ISSPITTING,d
   jr nz,MSU_ContSpitting
MSU_NotSpitting:
   ld hl,SOLSTRUCT+XPOS         ;we only want to spit when Sol is close
   ld d,20                      ;range above or below where enemy will spit
   ld a,b                       ;\
   add a,d                      ; |
   cp (hl)                      ; | return if sol is not in range 
   ret c                        ; |  of the enemy
   ld a,(hl)                    ; |
   add a,d                      ; |
   cp b                         ; |
   ret c                        ;/
   set ISSPITTING,(ix+FLAGS)
   ret
MSU_ContSpitting:
   ld hl,&SpitterAni
   call &SpriteAnim
   dec c \ dec c                ;assume can continue moving up
   ld a,c                       ;\  must be at a position where (y pos)%8==0
                                ; | to check above, else keep going up
   and 7                        ;/ 
   call z,&CheckSparkAbove      ;hl, de set to tile addr, mapped pos
   ret nc                       ;ok to keep moving so return
   call nz,&MS_RemoveBlock      ;call the MoveSkull remove block routine
                                ; to remove a block if block was spit on
                                ;if z set, spitting at border of screen,
                                ;  so there is no block to remove
   call &GetPositionFromExtra
   inc c \ inc c                ;\ move down so starts spitting at
   res ISSPITTING,(ix+FLAGS)    ;done spitting so reset flag
   ret

MoveSpitterDown:
   ld hl,&EmptyAni
   call &SpriteAnim
   ld d,(ix+FLAGS)
   bit ISINITIALIZED,d
   jr nz,MSD_AlreadySetup
   call &SetupSpitter
MSD_AlreadySetup:
   bit ISSPITTING,d
   jr nz,MSD_ContSpitting
MSD_NotSpitting:
   ld hl,SOLSTRUCT+XPOS         ;we only want to spit when Sol is close
   ld d,20                      ;range above or below where enemy will spit
   ld a,b                       ;\
   add a,d                      ; |
   cp (hl)                      ; | return if sol is not in range 
   ret c                        ; |  of the enemy
   ld a,(hl)                    ; |
   add a,d                      ; |
   cp b                         ; |
   ret c                        ;/
   set ISSPITTING,(ix+FLAGS)
   ret
MSD_ContSpitting:
   ld hl,&SpitterAni
   call &SpriteAnim
   inc c \ inc c                ;assume can continue moving down
   ld a,c                       ;\  must be at a position where (x pos)%8==2
   sub 2                        ; | to check above, else keep going up
   and 7                        ;/ 
   call z,&CheckSparkBelow      ;hl, de set to tile addr, mapped pos
   ret nc                       ;ok to keep moving so return
   call nz,&MS_RemoveBlock      ;call the MoveSkull remove block routine
                                ; to remove a block if block was spit on
                                ;if z set, spitting at border of screen,
                                ;  so there is no block to remove
   call &GetPositionFromExtra
   res ISSPITTING,(ix+FLAGS)    ;done spitting so reset flag
   ret


;input:
;  a has char num
;destroys:
;  af,hl
SetupSpitter:
   push de
   ld d,(ix+FLAGS)
   sub FIRSTSPITTER             ;a now has 0-3 depending on which spitter
   rlca \ rlca \ rlca           ;mult by 8 since 8 bytes for each sprite
   ld hl,&FirstSpitter
   ld d,0
   ld e,a
   add hl,de                    ;hl has addr of sprite to draw
   call &AllignedSprite         ;draw spitter into the background
   set ISINITIALIZED,(ix+FLAGS) ;the enemy is now initialized
   pop de
   ret




;Description:
;  Set the position of the skull to value in EXTRA
;   and changes character from SKULLSETUP to SKULL
;CAUTION: doesn't have a RET at the end
;Input:
;  b,c has the x,y coordinate
;  ix has the structure of the enemy
;CALLS MoveSkull!
MoveSkullSetup:
   call &GetPositionFromExtra   ;get original x,y position
   call &GetHLSprite            ;hl has addr of tile (put a "WINDOW" here)
   ld (hl),WINDOW               ;put the tile window tile in the map
   ld hl,&Window
   call &AllignedSprite         ;draw the window in background screen
   ld (ix+CHARNUM),SKULL        ;change the charnum from setup to regular skull
   inc b                        ;start in middle of 8 pixels
   ;now do the move skull

;Input:
;  b,c has the x,y coordinate
;  ix has the structure
;Output:
;  b,c has the new x,y coordinate
;  SPRTADDR updated
MoveSkull:
   ld a,(ix+FLAGS)              ;bit 7 has direction moving (L or R)
                                ;low 7 bits have current "life" remaining
   inc a                        ;inc life
   ld (ix+FLAGS),a
   ld e,%01111111               ;mask to only look at "life"
   and e
   cp e                         ;are we at end of life
   jr z,MS_Reset                ;yes, so reset
   rrca                         ;put the low bit (which switches every
                                ; MoveSkull call) in c flag
   jr c,MS_KeepSameSprite       ;only update sprite every other MoveSkull call
   ld a,(ix+OFFSET)             ;ld sprite offset in a
   sub 9                        ;spriteanim will set the same sprite to use
   ld (ix+OFFSET),a             ;save the increment
MS_KeepSameSprite:
   ld hl,&SkullAni
   call &SpriteAnim             ;get the hl of the sprite to draw
   ld a,c                       ;\  must be at a position where (y pos)%8==0
   and 7                        ;/ to check below, else keep going down
   call z,&CheckDown
   jr c,MS_NoMoveDown
MS_MoveDown:
   inc c
   ret
MS_NoMoveDown:
   bit 7,(ix+FLAGS)             ;is it moving left?
   jr z,MS_MoveRight
MS_MoveLeft:
   dec b                        ;assume can continue moving left
   ld a,b                       ;\  must be at a position where (x pos)%8==0
                                ; | to check left, else keep going left
   and 7                        ;/ 
   call z,&CheckSparkLeft
   ret nc                       ;ok to keep moving so return
   inc b                        ;restore original x position
   res 7,(ix+FLAGS)             ;move right next time
   jr MS_MoveNewDirection
MS_MoveRight:
   inc b                        ;assume can continue moving right
   ld a,b                       ;\  must be at a position where (x pos)%8==2
   sub 2                        ; | to check right, else keep going right
   and 7                        ;/ 
   call z,&CheckSparkRight
   ret nc                       ;ok to keep moving so return
   dec b                        ;restore original x position
   set 7,(ix+FLAGS)             ;move left next time
;try to remove a block if this is what caused the turn around
;bc has position of skull
;de has the mapped position of tile to try to remove
;hl has address of tile to try to remove
MS_MoveNewDirection:
   ld (ix+DELAY),11             ;wait awhile before turning around
MS_RemoveBlock:
   ld a,121                     ;\
   cp b                         ; | if character at the borders of the screen
   ret z                        ; |  there can't be a block to remove
   ld a,1                       ; |  so return
   cp b                         ; |
   ret z                        ;/
   ld a,(hl)
   cp WALL                      ;is this the wall tile?
   ret z                        ;yep, so it is not removed... only blocks can be.
   push bc
   and %00111111                ;mask 2 most sig. bits to reveal tile below
   ld (hl),a                    ;save tile in map
   call &SpriteLookUp           ;get the sprite addr of tile in a
   call &GetPosition            ;bc has position to draw sprite
   call &AllignedSprite         ;draw the sprite
   pop bc                       ;restore position of the enemy
   ret
MS_Reset:
   ld (ix+CHARNUM),SKULLSETUP   ;call the setup function again
   ld a,(ix+FLAGS)
   and %10000000                ;reset the life of enemy
   ld (ix+FLAGS),a
   ld hl,&EmptyAni
   ld (ix+OFFSET),0
   call &SpriteAnim             ;get the hl of the sprite to draw
   ld (ix+DELAY),255
   ld bc,$F0F0                  ;move out of screen so until ready to move again
   ret

;description:
;  get the x,y position from an enemy based on the EXTRA byte
;input:
;  ix - enemy struct
;output:
;  bc has x,y position
;destroys:
;  af, de
GetPositionFromExtra:
   ld a,(ix+EXTRA)
   ld d,a
   and %00001111                ;only look at y pos
   ld e,a                       ;mapped y in e
   ld a,d
   and %11110000                ;only look at x pos
   rrca \ rrca \ rrca \ rrca    ;put in low nibble
   ld d,a                       ;mapped x in d
   call &GetPosition            ;get the x,y position in b,c from mapped in d,e
   ret


;description:
;  change the sparks that start out above or on the left side of a block
;  to the bottom, right side, it can then be moved with MoveSpark routine
;Input:
;  a has character number
;  b,c has the x,y coordinate
;  ix has the structure of the spark
;CALLS MOVESPARK!
ChangeSpark:
   sub 2                        ;change char number to a normal spark
   ld (ix+CHARNUM),a            ; so next time it will call MoveSpark
   inc b \ inc b                ;\ move to correct postion for sprite
   inc c \ inc c                ;/
   ld (ix+FLAGS),2              ;moving left on top of block
   call &MoveSpark
   ret

;Input:
;  b,c has the x,y coordinate
;  ix has the structure of the spark
;Output:
;  b,c has the new x,y coordinate
;  SPRTADDR updated
;destroys:
;  a,de,hl
MoveSpark:
;   dec (ix+DELAY)
;   ret nz                       ;not ready to move spark yet
MoveSparkNoDelay:               ;(entry point that fireball uses)
   ld a,(ix+FLAGS)              ;a has 0-3 (move right on bottom of block,
                                ; Down on Left, Left on Top, and Up on Right
   and 3                        ;only look at 2 Least significant bits
   dec a
   jr z,MSDonL                  ;1?
   dec a
   jr z,MSLonT                  ;2?
   dec a
   jr z,MSUonR                  ;3?
   ;else 0 so move right on bottom

MSRonB:
   ld a,b                       ;\ must be at a position where (x pos)%8==0
   and 7                        ;/  to check above, else keep going right
   jr nz,MSRonBCheckRight
   call &CheckSparkAbove
   jr c,MSRonBmove              ;can't move up so move right
   dec hl                       ;check above and one tile behind you
   bit 7,(hl)                   ;can spark move here too?
   jr z,MSRonBmove              ;yes so block disappeared, don't do air circle
   jr nz,MSUonRmove             ;no, so move Up around the tile
MSRonBCheckRight:
   ld a,b                       ;\  must be at a position where (x pos)%8==2
   sub 2                        ; | to check right, else keep going right
   and 7                        ;/  ("and" resets carry so wont branch)
   call z,&CheckSparkRight
   jr c,MSDonLmove              ;can't move right so move down
MSRonBmove:
   inc b                        ;else keep moving right
   ld a,SPARKRIGHT              ;moving right on bottom
   jr DoneMovingSpark

MSDonL:
   ld a,c                       ;\ must be at a position where (y pos)%8==0
   and 7                        ;/  to check right, else keep going down
   jr nz,MSDonLCheckBelow
   call &CheckSparkRight
   jr c,MSDonLmove              ;can't move right so move down  
   ld de,-16                    ;\ check right and one tile above
   add hl,de                    ;/
   bit 7,(hl)
   jr z,MSDonLmove
   jr nz,MSRonBmove
MSDonLCheckBelow:
   ld a,c                       ;\  must be at a position where (y pos)%8==2
   sub 2                        ; | to check below, else keep going down
   and 7                        ;/  ("and" resets carry so wont branch)
   call z,&CheckSparkBelow
   jr c,MSLonTmove
MSDonLmove:
   inc c
   ld a,SPARKDOWN
   jr DoneMovingSpark

MSLonT:
   ld a,b                       ;\ 
   sub 2                        ; |must be at a position where (x pos)%8==2
   and 7                        ;/  to check below, else keep going left
   jr nz,MSLonTCheckLeft
   call &CheckSparkBelow
   jr c,MSLonTmove              ;can't move below so move left
   inc hl                       ;check below and one tile behind you
   bit 7,(hl)                   ;can spark move here too?
   jr z,MSLonTmove              ;yes so block disappeared, don't do air circle
   jr nz,MSDonLmove             ;no, so move down around the tile
MSLonTCheckLeft:
   ld a,b                       ;\  must be at a position where (x pos)%8==0
                                ; | to check left, else keep going left
   and 7                        ;/  ("and" resets carry so wont branch)
   call z,&CheckSparkLeft
   jr c,MSUonRmove
MSLonTmove:
   dec b
   ld a,SPARKLEFT
   jr DoneMovingSpark

MSUonR:
   ld a,c                       ;\ 
   sub 2                        ; |must be at a position where (y pos)%8==2
   and 7                        ;/  to check left, else keep going up
   jr nz,MSUonRCheckAbove
   call &CheckSparkLeft
   jr c,MSUonRmove              ;can't move left so move up
   ld de,16                     ;\ check left and one tile below
   add hl,de                    ;/
   bit 7,(hl)
   jr z,MSUonRmove
   jr nz,MSLonTmove
MSUonRCheckAbove:
   ld a,c                       ;\  must be at a position where (y pos)%8==0
                                ; | to check above, else keep going up
   and 7                        ;/  ("and" resets carry so wont branch)
   call z,&CheckSparkAbove
   jr c,MSRonBmove
MSUonRmove:
   dec c
   ld a,SPARKUP

DoneMovingSpark:
   ld (ix+FLAGS),a              ;load direction spark is moving
   ld hl,&SparkAni
   call &SpriteAnim
   ld a,(ix+CHARNUM)
   cp FASTSPARK                 ;is this the fast spark character?
   ret nz                       ;no, so return
   dec (ix+DELAY)               ;it's fast, so it moves 2X faster
   ret


;input:
;  b,c is spark's x,y pos
;Output:
;  if c flag is set, cannot move
;    - and if z flag is reset (nonZero), could not move due to tile
;    - and if z flag is set (Zero), could not move because at screen border
;  if c flag is reset, can move
;  hl has address of tile if not at screen border
;  de has mapped position of tile if not at screen border
;destroys:
;  af
;ret a == 0 if failed because too far up
CheckSparkAbove:
   ld a,c                       ;\
   or a                         ; |at top of screen? (y pos == 0)
   scf                          ; | if so cannot move up
   ret z                        ;/
   call &GetMapped              ;mapped position in d,e
   dec e                        ;mapped of tile above
   call &GetHLSprite            ;get the address in the map of tile above
   ld a,(hl)
   rlc a                        ;if this is a solid tile, put high bit
                                ; in c flag and reset the z flag to NZ
   ret
;ret a == 50 if failed because too far down
CheckSparkBelow:
   ld a,c                       ;\
   cp 50                        ; |at lowest position on screen? (y pos==50)
   scf                          ; | if so cannot move down
   ret z                        ;/
   call &GetMapped              ;mapped position in d,e
   inc e                        ;mapped of tile below
   call &GetHLSprite            ;get the address in the map of tile below
   ld a,(hl)
   rlc a                        ;if this is a solid tile, put high bit
                                ; in c flag and reset the z flag to NZ
   ret
;ret a == 122 if failed because too far right
CheckSparkRight:
   ld a,b                       ;\
   cp 122                       ; |at rightmost pos. on screen? (x pos==122)
   scf                          ; | if so cannot move right
   ret z                        ;/
   call &GetMapped              ;mapped position in d,e
   inc d                        ;mapped of tile to right
   call &GetHLSprite            ;get the address in the map of tile to right
   ld a,(hl)
   rlc a                        ;if this is a solid tile, put high bit
                                ; in c flag and reset the z flag to NZ
   ret
;ret a == 0 if failed because too far left
CheckSparkLeft:
   ld a,b                       ;\
   or a                         ; |at left of screen? (x pos == 0)
   scf                          ; | if so cannot move left
   ret z                        ;/
   call &GetMapped              ;mapped position in d,e
   dec d                        ;mapped of tile to left
   call &GetHLSprite            ;get the address in the map of tile to left
   ld a,(hl)
   rlc a                        ;if this is a solid tile, put high bit
                                ; in c flag and reset the z flag to NZ
   ret


;Input:
;  a has character number
;  b,c has the x,y coordinate
;  ix has the structure of the flame
;Output:
;  b,c has the new x,y coordinate
;  SPRTADDR updated
;destroys:
;  af,de,hl
MoveFlame:
   call &GetMapped              ;mapped flame position in de
   ld hl,&FlameAni              ;assume flame is not low
   bit ISFLAMEFALLING,(ix+FLAGS);is the flame falling
   jr nz,MF_FinishUp            ;yes, don't worry about wanding then
   bit ISFLAMELOW,(ix+FLAGS)    ;is the flame low?
   jr z,MF_CheckForLow          ;no, so check if it should be low
   ld hl,&FlameLowAni           ;else we will use the FlameLow animation
   dec (ix+EXTRA)               ;decrement the low flame count
   call nz,&SpriteAnim          ;use FlameLowAni since still low and return
   ret nz                       ; (SpriteAnim - does not destroy flag register)
   ;else low flame is now over
   dec c \ dec c \ dec c        ;move back up
   res ISFLAMELOW,(ix+FLAGS)
   ld hl,&FlameAni
MF_CheckForLow:
   ld a,(SOLSTRUCT+FLAGS)
   bit ISWANDING,a              ;is sol using his wand?
   jr z,MF_FinishUp             ;nope, so use FlameAni animation
   ;check to see if sol's wand position is same as the flame's position
   push hl                      ;save animation
   ld hl,(LASTWANDTRY)          ;mapped wand position in hl
   or a                         ;clear the carry flag
   sbc hl,de                    ;are the mapped positions the same
   pop hl                       ;restore animation
   jr nz,MF_FinishUp            ;not in same position, so don't use low flame
   ;else sol is wanding at the low flame
   inc c \ inc c \ inc c        ;move the flame down so it's easier to jump
   ld hl,&FlameLowAni
   ld (ix+EXTRA),75             ;set the low flame counter
   set ISFLAMELOW,(ix+FLAGS)
MF_FinishUp:
   call &SpriteAnim
   res ISFLAMEFALLING,(ix+FLAGS);assume not falling
   call &CheckDown              ;c flag set if cannot fall
   ret c                        ;cannot fall so return
   set ISFLAMEFALLING,(ix+FLAGS);assumption wrong
   inc c                        ;can fall
   ret



;input:
;  ix has sol struct
;  b,c has sol's x,y
;destroys:
;  af,ix
StartFireball:
   ld ix,FBSTRUCT
   ld a,(ix+EXTRA)              ;extra used to time life span of fireball
   or a
   ret nz                       ;time is not zero so fireball still in action
   ld hl,NUMFIREBALLS
   xor a
   cp (hl)
   ret z                        ;no fireballs in scroll so nothing to do
   dec (hl)                     ;one less fireball
   ld (ix+EXTRA),FIREBALLLIFE   ;lifespan of fireball in EXTRA field
   ld (ix+DELAY),1              ;fireball will be moved if delay not 0
   ld a,b
   inc a                        ;move x position more towards center of Sol
   inc a
   and %11111000                ;starting X pos divisible by 8
   ld (ix+XPOS),a
   push bc
   call &DrawScroll             ;redraw the scroll with 1 less fireball
   pop bc
   ld (ix+FLAGS),SPARKRIGHT     ;assume sol facing left
   ld a,c
   bit 7,a                      ;is sol's y position < 0 (i.e. negative)
   jr z,SF_NonNegY              ;yes, so branch
   xor a                        ;make fireball's init. y pos. = 0
SF_NonNegY:
   and %11111110                ;starting Y pos divisible by 2
   ld (ix+YPOS),a
   ld a,(SOLSTRUCT+FLAGS)
   bit ISLEFT,a
   ret z                        ;assumption correct so return
   ld (ix+FLAGS),SPARKLEFT
   inc (ix+YPOS)                ;move the y position down 2 pixels if left
   inc (ix+YPOS)
   ret



;input:
;  none
;output:
;  updates fbstruct
;destroys:
;  all
MoveFireball:
   ld ix,FBSTRUCT
   ld a,(ix+EXTRA)
   or a
   ret z                        ;no fireball in action
   dec (ix+DELAY)               ;only move FB when decrement is 0
   ret nz 
   dec (ix+EXTRA)
   ld b,(ix+XPOS)
   ld c,(ix+YPOS)
   call &MoveSparkNoDelay       ;use the spark's move function
   ld e,3                       ;size of box for collision
   call &TestEnemyCollision
   jr c,NoFBCollision
   ;d has enemy # collided with
   ;hl has addr of y pos of enemy collided with
   ;this code moves the enemies "down" SIZEOFSTRUCT bytes to cover killed enemy
KillEnemy:
   push bc

   or a
   ld bc,YPOS
   sbc hl,bc                    ;hl = addr of enemy struct (c flag = 0)

   ;!!!! CHARNUM !!!!
   ;CAUTION: not going to use the solstruct constant here
   ld a,(hl)
   cp SKULL                     ;is this the SKULL character?
   jr nz,KillEnemyNotSkull      ;no! so jump

   ;skull character does not get removed from enemy structs
   ; instead it just gets reset to its original state

   push ix                      ;save FBstruct
   push hl
   pop ix                       ;enemy struct needs to be in ix
   call &MS_Reset               ;reset this enemy
   ld (ix+XPOS),b               ;\ needs to be saved!
   ld (ix+YPOS),c               ;/
   pop ix                       ;FB struct back in ix
   jr KillEnemyFinishUp

KillEnemyNotSkull:
   cp FIRSTSPITTER
   jr c,KillRegularEnemy
   cp LASTSPITTER+1
   jr nc,KillRegularEnemy
KillSpitter:
   push ix
   push hl
   push hl
   pop ix
   push de
   call &GetPositionFromExtra
   pop de
   ld hl,&Empty
   call &AllignedSprite
   pop hl
   pop ix


KillRegularEnemy:
   ld a,(NUMENEMIES)
   dec a
   sub d
   jr z,DecNumEnemies           ;last enemy in list so no copy needed
   ld b,a                       ;b has number of SIZEOFSTRUCT blocks to copy
   xor a
KillEnemyMult:
   add a,SIZEOFSTRUCT
   djnz KillEnemyMult           ;a has # of bytes to copy

   push hl
   pop de                       ;de destination addr to move enemies to

   ld bc,SIZEOFSTRUCT
   add hl,bc                    ;hl source addr to copy enemy structs from

   ld b,0
   ld c,a                       ;bc has # of bytes to copy

   ldir                         ;copy (de)=(hl) for bc times

DecNumEnemies:
   ld hl,NUMENEMIES
   dec (hl)
KillEnemyFinishUp:
   pop bc
   ld (ix+EXTRA),0              ;kill the fireball
NoFBCollision:
   ld (ix+XPOS),b
   ld (ix+YPOS),c
   ld hl,&FireballAni
   call &SpriteAnim
   ret





;Input:
;  none
;Output:
;  updates sol structure
;Destroys:
;  all
MoveSol:
   ld ix,SOLSTRUCT
   dec (ix+DELAY)               ;only move sol when decrement is 0
   ret nz 

   ld b,(ix+XPOS)               ;get sol's x pos
   ld c,(ix+YPOS)               ;get sol's y pos

   ld a,%00011110               ;\
   out (1),a                    ; | a has keypress (mask what i don't want)
   in a,(1)                     ;/

   push af
   call &MoveSolFall            ;does sol need to be falling?
   pop af

   call &LandRTest              ;both L and R can't be pressed at same time

   bit 0,a                      ;DOWN pressed
   call z,&DownMask             ;mask out certain keys

   call &WandingMask            ;mask out L and R if wanding

   push af
   cpl
   and %00000110
   call z,&SolStanding          ;neither LEFT nor RIGHT is pressed
   pop af

   bit 0,a                      ;DOWN pressed
   call z,&SolDucking

   push bc                      ;wanding can't change sol's x,y position
   push af
   bit ISWANDING,(ix+FLAGS)     ;is the wanding bit set?
   call nz,&MoveSolWandCont        
   pop af
   bit 5,a                      ;2ND pressed
   push af
   call z,&MoveSolWandStart
   pop af
   pop bc                       ;restore sol's x,y

   push af
   bit ISFBWANDING,(ix+FLAGS)   ;is the fireball bit set?
   call nz,&MoveSolFBCont
   pop af
   push af
   bit 7,a                      ;ALPHA pressed
   call z,&MoveSolFBStart
   pop af

   bit 1,a                      ;LEFT pressed
   push af
   call z,&MoveSolLeft
   pop af

   bit 2,a                      ;RIGHT pressed
   push af
   call z,&MoveSolRight
   pop af

   bit 3,a                      ;UP pressed
   push af
   bit ISJUMPING,(ix+FLAGS)     ;is sol in the middle of a jump?
   call nz,&MoveSolJumpCont     ;continue jump (must be before MSJS)
   call &MoveSolBreakCont       ;continue breaking block
   call &MoveSolHitHead         ;sol hits his head on block above (b4 MSJS)
   pop af
   push af
   call z,&MoveSolJumpStart     ;start jump
   pop af

   ld (LASTKEYS),a              ;save last keypress

   ld a,(ix+FLAGS)
   ld (LASTFLAGS),a             ;save sol's last flags

   call &SpriteAnim

   ld (ix+XPOS),b               ;save sol's x
   ld (ix+YPOS),c               ;save sol's y

   ld a,(SOLSTRUCT+FLAGS)
   bit ISFBWANDING,a
   call nz,&StartFireball

   call &PickUpItem

   ret


;input:
;  bc has sol's x,y position
;destroys:
;  a,bc,de,hl,ix
PickUpItem:
   inc b \ inc b \ inc b
   inc c \ inc c
   inc c \ inc c
   call &GetMapped
   call &GetHLSprite
   ld a,(hl)
   cp NUMITEMS                  ;can only use jump table for items 0 - N.I.-1
   ret nc                       ;number is too large (shouldn't ever occur)
   or a                         ;is map empty here?
   ret z                        ;yes, so nothing to do
   call &GetPosition            ;bc has x,y position to draw EmptySprite
   ld a,(hl)                    ;put item into a
   dec a                        ;start at item after empty
   ld ix,&ItemTable             ;jump table of items in map
   ld d,a                       ;\
   add a,a                      ; | 3 bytes for each jp instruction
   add a,d                      ;/   so mult a by 3
   ld d,0
   ld e,a
   add ix,de
   push ix                      ;put address on stack
   ret                          ;put address into PC
;for all of the jumps, hl has addr of tile in map and bc has tile position
ItemTable:
   jp &HourGlassItem
   jp &WallItem
   jp &LifeBottleItem
   jp &FireBallBottleItem
   jp &MoreScrollItem
   jp &WindowItem
   jp &ClosedDoorItem
   jp &KeyItem
   jp &OpenDoorItem
HourGlassItem:                  ;change game timer to 5000
   push hl
   ld de,(GAMETIMER)
   sla e
   rl d
   ld (GAMETIMER),de
   pop hl
   jr RemoveItemFromMap
WallItem:
   ret
LifeBottleItem:                 ;double game timer
   push hl
   ld hl,SOLLIVES
   inc (hl)
   pop hl
   jr RemoveItemFromMap
FireBallBottleItem:
   push hl
   push bc
   ld hl,NUMFIREBALLS
   ld a,(NUMSCROLL)
   inc (hl)                     ;assume scroll long enough to add a fireball
   cp (hl)                      ;was assumption correct?
   jr nc,FBBIcont               ;numscroll is >= so correct
   dec (hl)                     ;decrease count to what it was before
FBBIcont:
   call &DrawScroll             ;redraw scroll
   pop bc
   pop hl
   jr RemoveItemFromMap
MoreScrollItem:
   push hl
   push bc
   ld hl,NUMSCROLL
   ld a,MAXSCROLL
   inc (hl)                     ;assume scroll not longer than max
   cp (hl)
   jr nc,MSIcont                ;max size >= new size so branch
   dec (hl)                     ;decrease to what it was before
MSIcont:
   call &DrawScroll             ;redraw scroll
   pop bc
   pop hl
   jr RemoveItemFromMap
FairyBellItem:
   jr RemoveItemFromMap
BonusLevelItem:
   jr RemoveItemFromMap
WindowItem:
   ;do nothing
   ret
ClosedDoorItem:
   ;do nothing
   ret
;must have bc at sprite's location and hl point to tile in map
RemoveItemFromMap:
   ld (hl),0
   ld hl,&Empty
   call &AllignedSprite
   ret

KeyItem:
   push bc
   push hl
   ld hl,GAMEFLAGS
   set GOTKEY,(hl)
   ld hl,LEVEL-1
   ld e,-1
FindDoorLoop:
   inc e
   inc hl
   ld a,(hl)
   and %00111111                ;need to look if it is hidden behind block
   cp CLOSEDDOOR
   jr nz,FindDoorLoop
   ld a,(hl)
   and %11000000                ;save the 2 most significant bits
   ld d,OPENDOOR
   or d                         ;a now has sprite to draw
   ld b,a                       ;save sprite to draw in b
   ld (hl),a
   ld a,e
   call &GetMappedFromOffset    ;mapped position in d,e
   ld a,b                       ;sprite to draw in a for LookUp
   call &SpriteLookUp           ;hl has address of sprite to draw
   call &GetPosition            ;bc has position to draw sprite
   call &AllignedSprite
   pop hl
   pop bc
   jr RemoveItemFromMap

OpenDoorItem:
   ld hl,GAMEFLAGS
   set ISFINISHED,(hl)          ;finished level!
   ret


;destroys all registers
OpenTheDoor:
   ld hl,LEVEL-1
   ld e,-1
FindKeyLoop:   
   inc e
   inc hl
   ld a,(hl)
   and %00111111                ;check to see if key is hidden
   cp KEY
   jr nz,FindKeyLoop            ;loop until it is found
                                ; hl will have address of KEY
                                ; e will have offset from start of level
   ld a,e
   call &GetMappedFromOffset    ;mapped position in d,e
   call &GetPosition            ;position in b,c
   ;hl has addr of tile in map
   ;bc has location of tile
   jr KeyItem
   

;input:
;  a has keypress
;  ix has sol struct addr
;output:
;  hl has animation addr
;destroys:
;  a, de, bc
MoveSolWandStart:
   ex de,hl
   ld hl,LASTKEYS               ;\
   bit 5,(hl)                   ;/return until lets go of 2nd
   ex de,hl
   ret z
   set ISWANDING,(ix+FLAGS)
   inc c \ inc c \ inc c
   inc b 
   bit ISLEFT,(ix+FLAGS)
   jr nz,GetLeftPosition        ;facing left so get x position to use
   inc b \ inc b \ inc b        ;else right
   inc b \ inc b
   call &GetMapped              ;de has mapped position of sol
   call &GetHLSprite            ;\
   bit 7,(hl)                   ; | shitty code to remove a bug
   jr nz,Bug1                   ;/
   inc d                        ;move one right
Bug1:
   ld hl,&SolWandR1             ;use first wanding animation
   bit 0,a                      ;is sol ducking? (down pressed)
   jr nz,GotMappedBlock         ;sol isn't ducking, so branch
   inc e                        ;move down one since ducking
   ld hl,&SolSqtWandR1          ;use first ducking, wanding animation
   jr GotMappedBlock
GetLeftPosition:
   call &GetMapped              ;de has mapped position to put block
   call &GetHLSprite            ;\
   bit 7,(hl)                   ; | shitty code to remove a bug
   jr nz,Bug2                   ;/
   dec d                        ;move one left
Bug2: 
   ld hl,&SolWandL1             ;use first wanding animation
   bit 0,a                      ;is sol ducking? (down pressed)
   jr nz,GotMappedBlock         ;sol isn't ducking, so branch
   inc e                        ;move down one since ducking
   ld hl,&SolSqtWandL1          ;use first ducking, wanding animation
GotMappedBlock:
   push hl                      ;save animation addr
   ld (LASTWANDTRY),de          ;save this try with the wand
   ld a,d
   cp -1                        ;block too far right?
   jr z,CantWandBlock
   cp 16                        ;block too far left?
   jr z,CantWandBlock
   ld a,e
   cp 7                         ;block too far down?
   jr z,CantWandBlock
   call &GetPosition            ;get the x,y position in bc
   call &GetHLSprite            ;hl has address of sprite in map
   ld a,(hl)                    ;a has tile number
   or a                         ;is tile in map empty? (0)
   jr z,PutNewBlock             ;yes, so put a new block
   bit 7,a                      ;is a block there? (destroy block)
   jr nz,WandBlock              ;yes, so branch (else cant use wand)
CantWandBlock:
   ld de,-1
   ld (WANDXY),de
   pop hl                       ;restore animation
   ret
PutNewBlock:
   push de
   push hl
   ld e,8                       ;size of box for collision
   call &TestEnemyCollision
   pop hl
   pop de
   jr nc,CantWandBlock
   ld (hl),BLOCK
   ld hl,&BlockStar
   jr FinishStartingWand
WandBlock:
   cp WALL                      ;is this the Wall tile?
   jr z,CantWandBlock           ;yes, so can't use wand
   and %00111111                ;mask 2 most sig. bits to reveal tile below
   ld (hl),a                    ;save tile in map
   ld hl,&BlockBreak
FinishStartingWand:
   ld (WANDXY),de
   call &AllignedSprite
   pop hl                       ;restore animation
   ret


MoveSolWandCont:
   res ISWANDING,(ix+FLAGS)
   ld de,(WANDXY)
   ld a,d
   cp -1                        ;is no new sprite being drawn?
   jr z,GetWandContAni          ;nothing new so get the animation
   call &GetHLSprite            ;get the address of the sprite in the map
   ld a,(hl)                    ;put the tile in a
   ld hl,&Block
   cp BLOCK                     ;is the sprite the block?
   jr z,Draw2ndWanding          ;yes, so ready to draw, else....
   call &SpriteLookUp           ;get the address of the sprite to draw from a
Draw2ndWanding:
   call &GetPosition
   call &AllignedSprite
GetWandContAni:
   ld a,(LASTKEYS)              ;was down pressed when wanding started?
   bit 0,a
   jr z,DuckingWandingCont      ;yes, so branch
   ld hl,&SolWandL2             ;assume facing left
   bit ISLEFT,(ix+FLAGS)
   ret nz                       ;facing left
   ld hl,&SolWandR2
   ret
DuckingWandingCont:
   ld hl,&SolSqtWandL2          ;assume facing left
   bit ISLEFT,(ix+FLAGS)
   ret nz                       ;facing left
   ld hl,&SolSqtWandR2
   ret


;input:
;  a has keyboard input
;  ix has sol struct addr
;output:
;  hl has animation addr
;destroys:
;  a, de
MoveSolFBStart:
   ex de,hl
   ld hl,LASTFLAGS
   bit ISFBWANDING,(hl)         ;return if wanding
   ex de,hl
   ret nz
   ex de,hl
   ld hl,LASTKEYS               ;\
   bit 7,(hl)                   ;/return until lets go of Alpha
   ex de,hl
   ret z
   set ISFBWANDING,(ix+FLAGS)
   bit 0,a                      ;is sol ducking
   jr z,DuckingWandFBCont       ;yes, so branch
   ld hl,&SolWandL1             ;assume facing left
   bit ISLEFT,(ix+FLAGS)
   ret nz                       ;facing left
   ld hl,&SolWandR1
   ret
DuckingWandFBCont:
   ld hl,&SolSqtWandL1          ;assume facing left
   bit ISLEFT,(ix+FLAGS)
   ret nz                       ;facing left
   ld hl,&SolSqtWandR1
   ret

MoveSolFBCont:
   res ISFBWANDING,(ix+FLAGS)   ;no longer using wand to throw fireball
   jr GetWandContAni            ;same second animation as wanding


;input:
;  bc has the x,y position of the object to test for collision
;  e has the collision box size
;output:
;  no carry if collision occurs
;  carry set if no collision
;  d has the enemy number (0 thru NUMENEMIES-1) *if* collision occurs
;  hl has addr of y pos of enemy in ENEMYSTRUCTS *if* collision occurs
;destroys:
;  de,af
TestEnemyCollision:
   ld a,(NUMENEMIES)
   or a
   scf                          ;set no collision if 0 enemies
   ret z                        ;return if zero enemies
   ld d,0                       ;start at first enemy (#0)
   ld hl,ENEMYSTRUCTS+XPOS      ;address of first enemy's x position
TECLoop:
   ld a,e                       ;size of sprites for SpriteCollision routine
   push de
   ld d,(hl)                    ;load x pos of enemy
   inc hl
   ld e,(hl)                    ;load y pos of enemy
   call &SpriteCollision
   pop de                       ;d=enemy #, e=box collision size
   ret nc                       ;collision with enemy
   push de                      ;save enemy number and box size
   push hl                      ;save address of y position of this enemy
   push bc                      ;save x,y position
   push ix
   call &TestSpitterExtra       ;check to see if there is a collision with spitter
   pop ix
   pop bc                       ;restore x,y position
   pop hl                       ;restore address of y position of this enemy
   pop de                       ;restore enemy number and box size
   ret nc                       ;collision with the spitter's location in EXTRA
   push de
   ld de,SIZEOFSTRUCT - 1       ;move to next enemy
   add hl,de
   pop de
   inc d                        ;move to next enemy
   ld a,(NUMENEMIES)
   cp d
   jr nz,TECLoop                ;jump if not at last enemy
   scf                          ;no collision so set carry
   ret


;input:
;  bc has the x,y position of the object to test for collision
;  hl has address of enemy struct's y position
;  e has the collision box size
;output:
;  no carry if collision occurs
;  carry set if no collision
;destroys:
;  af,bc,de,hl,ix
TestSpitterExtra:
   ld a,e
   ld de,0 - YPOS
   add hl,de                    ;hl has address of enemy structure
   ld e,a                       ;save collision box size
   ld a,(hl)                    ;get the charnum
   cp FIRSTSPITTER              ;is CHARNUM less than FIRSTSPITTER?
   ret c                        ;yes, so it's not spitter, return
   ld a,LASTSPITTER
   cp (hl)                      ;is CHARNUM greater than LASTSPITTER?
   ret c                        ;yes, so it's not spitter, return
   push hl
   pop ix                       ;ix has enemy structure
   ld a,e                       ;a has the collision box size
   ld d,b
   ld e,c                       ;put x,y position in d,e
   push af
   push de
   call &GetPositionFromExtra   ;get spitter enemy's position in b,c
   pop de
   pop af
   call &SpriteCollision        ;check for a collision (c set if no collision)
   ret




;Input:
;  bc has x,y position
;  ix has sol structure
;Output:
;  hl has sprite animation
SolDucking:
   set ISDUCKING,(ix+FLAGS)
   ld hl,&SolDuckL
   bit ISLEFT,(ix+FLAGS)
   ret nz
   ld hl,&SolDuckR
   ret



;Input:
;  bc has x,y position
;  ix has sol structure
;Output:
;  hl has sprite animation
;Destroys:
;  de
MoveSolJumpStart:
   bit ISFALLING,(ix+FLAGS)     ;is sol falling?
   ret nz                       ;yep, so he can't jump
   bit ISJUMPING,(ix+FLAGS)     ;already jumping?
   ret nz                       ;yes, so dont start new jump
   bit ISHITHEAD,(ix+FLAGS)     ;is sol hitting his head?
   ret nz                       ;yes, so don't start new jump
   ld a,(LASTKEYS)              ;\
   bit 3,a                      ; | return if holding down the up key
   ret z                        ;/
   set ISHITHEAD,(ix+FLAGS)     ;assume you are going to hit your head

   push bc
   inc b                        ;\
   inc b                        ; | check the middle of sol's head before left
   inc b                        ;/  and right corners
   call &GetMapped
   pop bc
   GETABOVELEFT
   ld a,e                       ;\
   cp -1                        ; | if above is -1, past top of screen,
   jr z,JumpAnimation           ;/    so hit your head
   call &GetHLSprite
   bit 7,(hl)                   ;is high bit set (cannot jump)?
   jr nz,JumpAnimation          ;cannot jump, so hit head

   inc b
   call &GetMapped              ;de has mapped x,y coordinates
   dec b
   GETABOVELEFT
   call &GetHLSprite
   bit 7,(hl)                   ;is high bit set (cannot jump)?
   jr nz,JumpAnimation          ;cannot jump, so hit head

   dec b \ dec b
   call &GetMapped              ;de has mapped x,y coordinates
   inc b \ inc b
   GETABOVERIGHT
   call &GetHLSprite
   bit 7,(hl)                   ;is high bit set (cannot jump)?
   jr nz,JumpAnimation

   res ISHITHEAD,(ix+FLAGS)     ;did not hit head.  it's ok to jump
   set ISJUMPING,(ix+FLAGS)     ;sol is starting jump
   xor a                        ;\
   ld (JUMPOFFSET),a            ;/ new jump so start at the beginning
JumpAnimation:
   ld (BLOCKXY),de              ;save the mapped position of block to remove
   ld (ix+OFFSET),0             ;go to beginning of animation
   ld hl,&SolCrouchL            ;assume jumping left
   bit ISLEFT,(ix+FLAGS)        ;is so facing left?
   ret nz                       ;yep, so return
   ld hl,&SolCrouchR            ;nope, facing right
   ret



MoveSolBreakCont:
   bit ISBREAKING,(ix+FLAGS)    ;is sol breaking a block?
   ret z                        ;no, so return
   push hl                      ;save sprite animation
   push bc                      ;save the sol's x,y
   res ISBREAKING,(ix+FLAGS)    ;not breaking anymore
   ld de,(BLOCKXY)              ;de has mapped sprite x,y
   call &GetHLSprite            ;hl has sprite address in map
   bit 6,(hl)                   ;is it a cracked or uncracked block?
   set 6,(hl)                   ;assume uncracked... now cracked
   jr z,DrawFinalSprite         ;if assumption correct, jump
   ;else the block already cracked
   ld a,(hl)
   and %00111111                ;mask out 2 high bits to reveal sprite below
   ld (hl),a                    ;store back into map
DrawFinalSprite:
   call &GetPosition            ;bc has x,y to draw sprite at
   ld a,(hl)                    ;put sprite to lookup in reg a
   call &SpriteLookUp           ;get the address of the sprite in reg. a
   call &AllignedSprite
   pop bc                       ;restore sol's x,y
   pop hl                       ;restore sprite animation
   ret


;Input:
;  bc has x,y position
;  ix has sol structure
;Output:
;  c (y pos) is incremented
;  hl has sprite animation
;Destroys:
;  de
MoveSolHitHead:
   bit ISHITHEAD,(ix+FLAGS)
   ret z                        ;not hitting head, so return
   res ISHITHEAD,(ix+FLAGS)     ;done hitting head
   ld de,(BLOCKXY)              ;de has mapped x,y of the block
   ld a,e                       ;\
   cp -1                        ; | if above is -1, past top of screen,
   jr z,MoveSolNotUp            ;/    so nothing to break
   call &GetHLSprite
   ld a,WALL                    ;\ is the sprite above the wall?
   cp (hl)                      ;/  (cannot be broken)
   jr z,MoveSolUp               ;yes, so can't break
   push bc                      ;save the x,y coordinate of sol
   call &GetPosition            ;bc has x,y position to draw sprite
   bit 6,(hl)                   ;is this a cracked block or uncracked?
   ld hl,&BlockSmCrk            ;assume it is uncracked block
   jr z,GotNewBlockSpr          ;assumption correct
   ld hl,&BlockBreak            ;block already cracked so it's breaking
GotNewBlockSpr:
   set ISBREAKING,(ix+FLAGS)    ;breaking a block
   call &AllignedSprite
   pop bc                       ;restore the x,y coordinate of sol
MoveSolUp:
   dec c                        ;move up
   dec c
MoveSolNotUp:
   ld hl,&SolCrouchL            ;assume left
   bit ISLEFT,(ix+FLAGS)
   ret nz                       ;yep, left
   ld hl,&SolCrouchR
   ret



;Input:
;  bc has x,y position
;  ix has sol structure
;Output:
;  c (y pos) is incremented if not at end of jump
;  hl has sprite animation
;Destroys:
;  de
;  a
MoveSolJumpCont:
   bit ISWANDING,(ix+FLAGS)     ;is sol using the wand right now?
   ret nz                       ;yes, so don't move during jump
   bit ISFBWANDING,(ix+FLAGS)   ;is sol using the wand to throw fireball now?
   ret nz                       ;yes, so don't move during jump
   ld a,(LASTFLAGS)
   bit ISWANDING,a              ;in 2nd stage of using wand?
   ret nz
   bit ISFBWANDING,a
   ret nz
   push hl                      ;save incoming sprite anim
   ld hl,JUMPOFFSET
   ld a,(hl)
   inc (hl)                     ;look at next byte next time function called
   ld l,a
   ld h,0
   ld de,&JumpingArc
   add hl,de                    ;hl points to the # of pixels to jump
   ld a,(hl)
   pop hl
   cp $F0                       ;is the jump amount $F0 (end of jump) ?
   jr z,FinishedJump
   neg
   add a,c                      ;move sol # pixels in JumpingArc
   ld c,a
   jr GetJumpSprite
FinishedJump:
   res ISJUMPING,(ix+FLAGS)     ;finished jumping
   set ISFALLING,(ix+FLAGS)     ;start falling
GetJumpSprite:
   ld a,(ix+XPOS)
   cp b
   ret z                        ;did not move L or R so use standing anim
   ld hl,&SolJmpL               ;assume sol is jumping left
   bit ISLEFT,(ix+FLAGS)        ;is he facing left?
   ret nz                       ;yes, left, so return
   ld hl,&SolJmpR               ;else facing right
   ret


;Input:
;  bc has x,y position
;  ix has sol structure
;Output:
;  bc has updated x,y
;Destroys:
;  de, hl
MoveSolFall:
   bit ISJUMPING,(ix+FLAGS)     ;is sol jumping right now?
   ret nz                       ;yes, so don't worry about falling
   bit ISWANDING,(ix+FLAGS)     ;is sol using the wand right now?
   ret nz                       ;yes, so don't move during fall
   bit ISFBWANDING,(ix+FLAGS)   ;is sol using the fbwand right now?
   ret nz                       ;yes, so don't move during fall
   ld a,(LASTFLAGS)
   bit ISWANDING,a              ;in 2nd stage of using wand?
   ret nz
   bit ISFBWANDING,a            ;in 2nd stage of using fbwand?
   ret nz
   res ISFALLING,(ix+FLAGS)     ;assume sol is not falling
   call &CheckDown              ;c flag is reset if can fall
   ret c                        ;cannot fall so return
   set ISFALLING,(ix+FLAGS)     ;set the falling bit
   inc c
   inc c
   ret



;Input:
;  a has keypress
;  ix has sol struct
;Output:
;  a updated
;Destroys:
;  d
WandingMask:
   ld d,a
   or %00001110                 ;mask out left, right, up
   bit 5,a                      ;is 2ND pressed?
   ret z                        ;yes, so mask out L,R, Up
   bit ISWANDING,(ix+FLAGS)     ;is sol using the wand?
   ret nz                       ;yes, so return w/out L,R,& Up pressed
   bit 7,a
   ret z
   bit ISFBWANDING,(ix+FLAGS)
   ret nz
   ld a,d                       ;else, not using wand
   ret


;Input:
;  a has keypress
;Output:
;  a updated
DownMask:
   or %00001111                 ;L, R, and Up, Down not pressed
   bit ISFALLING,(ix+FLAGS)
   ret nz
   bit ISJUMPING,(ix+FLAGS)
   ret nz
   and %11111110                ;down pressed
   ret



;Input:
;  a has keypress
;Output:
;  if both left and right are pressed,
;  a changed to neither pressed
;Destroys:
;  none
LandRTest:
   push af                      ;save key press
   and %00000110                ;mask out all bits but 1 and 2
   or a                         ;if a is 0, both left and right are pressed
   jr z,LRBoth
   pop af                       ;restore key press
   ret
LRBoth:
   pop af                       ;restore key press
   or %00000110                 ;set bits 1 & 2 high (not pressed)
   ret



;Input:
;  ix has sol sprite structure
;  bc has x,y position
;Output:
;  b (x position) updated
;  hl has beginning address of sprite animation
;Destroys:
;  de
MoveSolLeft:
   call &CheckLeft              ;is it ok to move left
   jr c,NoMoveSolL              ;could not move
   dec b
   ld hl,&SolRunL               ;assume not falling
   bit ISFALLING,(ix+FLAGS)
   ret z                        ;not falling so return
   ld (ix+OFFSET),0             ;offset back to 0
   ld hl,&SolJmpL               ;use jump animation
   ret
NoMoveSolL:
   ld (ix+OFFSET),0             ;offset back to 0
   ld hl,&SolStndL
   ret


;Input:
;  ix has sol sprite structure
;Output:
;  b (x position) updated
;  hl has beginning address of sprite animation
;Destroys:
;  de
MoveSolRight:
   call &CheckRight
   jr c,NoMoveSolR              ;could not move
   inc b
   ld hl,&SolRunR               ;assume not falling
   bit ISFALLING,(ix+FLAGS)
   ret z                        ;not falling so return
   ld (ix+OFFSET),0             ;offset back to 0
   ld hl,&SolJmpR               ;use jump animation
   ret
NoMoveSolR:
   ld (ix+OFFSET),0             ;offset back to 0
   ld hl,&SolStndR
   ret


;Input:
;  b,c has coordinate
;Output:
;  if c flag is set, cannot move down
;  if c flag is reset, can move down
;Destroys:
;  de, a
CheckDown:
   dec b \ dec b
   call &GetMapped              ;d,e has mapped coordinates
   inc b \ inc b
   GETBELOWRIGHT
   call &GetHLSprite            ;hl has addr of sprite in map
   ld a,(hl)
   rlca                         ;put bit 7 into carry flag
   ret c                        ;if it's set, block below
   inc b
   call &GetMapped
   dec b
   GETBELOWLEFT
   call &GetHLSprite
   ld a,(hl)
   rlca                         ;put bit 7 into carry flag
   ret c                        ;if it's set, block below
   ld a,47                      ;\is y position 48+ ? (bottom of map)
   cp c                         ;/
   ret                          ;return c set if ypos is >= 48


;Input:
;  b,c has coordinate
;Output:
;  if c flag is set, cannot move up
;  if c flag is reset, can move up
;Destroys:
;  de, af
CheckUp:
   ld a,7                       
   and c                        ;if y pos not divisble by 8, *can* move up
   ret z                        ;c flag is reset so can move up

   ld a,c                       
   or a                         ;is x position 0?
   scf
   ret z                        ;yes, so can't move up, set carry

   dec b \ dec b
   call &GetMapped              ;d,e has mapped coordinates
   inc b \ inc b
   GETRIGHT
   call &GetHLSprite            ;hl has addr of sprite in map
   ld a,(hl)
   rlca                         ;put the high bit in carry flag 
   ret c                        ;if set, unable to move up
   inc b
   call &GetMapped
   dec b
   GETLEFT
   call &GetHLSprite
   ld a,(hl)
   rlca                         ;put the high bit in carry flag 
   ret                          ;if set, unable to move up


;input:
;  ix has sol sprite structure
;destroys:
;  a,hl
SolStanding:
   ld (ix+OFFSET),0             ;beginning of anim
   bit ISFALLING,(ix+FLAGS)
   jr nz,SSStanding             ;currently falling, use standing anim
   ld a,(LASTFLAGS)
   bit ISFALLING,a
   jr z,SSStanding              ;standing
   ld hl,&SolCrouchL            ;else, compress for landing
   bit ISLEFT,(ix+FLAGS)
   ret nz
   ld hl,&SolCrouchR
   ret
SSStanding:
   ld hl,&SolStndL              ;assume sol standing left
   bit ISLEFT,(ix+FLAGS)        ;is sol facing left?
   ret nz                       ;yes, so return
   ld hl,&SolStndR
   ret



;Input:
;  b,c has coordinate
;  ix has sprite structure
;Output:
;  if c flag is set, cannot move left
;  if c flag is reset, can move left
;  updates the ISLEFT of FLAGS
;Destroys:
;  af,de,hl
CheckLeft:
   set ISLEFT,(ix+FLAGS)             ;facing left
   ld a,b                       ;\is x position 0?
   or a                         ;/
   scf                          ;set carry for return
   ret z                        ;x pos is zero, can't move farther left
   inc c
   call &GetMapped
   dec c
   GETLEFT
   call &GetHLSprite
   ld a,(hl)
   rlca                         ;put the high bit in carry flag 
   ret c                        ;if set, unable to move left
   dec c
   call &GetMapped
   inc c
   GETBELOWLEFT
   call &GetHLSprite
   ld a,(hl)
   rlca                         ;put the high bit in carry flag 
   ret                          ;if set, unable to move left


;Input:
;  b,c has coordinate
;  ix has sprite structure
;Output:
;  if c flag is set, cannot move right
;  if c flag is reset, can move right
;  updates the ISLEFT of FLAGS
;Destroys:
;  de,hl
CheckRight:
   res ISLEFT,(ix+FLAGS)        ;facing right
   ld a,119                     ;\is y position 120+ ? (right of map)
   cp b                         ;/
   ret c                        ;return c set if xpos is >= 120
   dec b
   inc c
   call &GetMapped
   dec c
   inc b
   GETRIGHT
   call &GetHLSprite
   ld a,(hl)
   rlca                         ;put the high bit in carry flag 
   ret c                        ;if set, unable to move right
   dec b
   dec c
   call &GetMapped
   inc c
   inc b
   GETBELOWRIGHT
   call &GetHLSprite
   ld a,(hl)
   rlca                         ;put the high bit in carry flag 
   ret                          ;if set, unable to move right



InitLevel:
   ld ix,ENEMYSTRUCTS           ;ix used to point to the next enemy in list
   xor a
   ld (NUMENEMIES),a            ;0 enemies
   inc a
   ld (REFRESHCOUNT),a          ;start refresh at 1

   ld de,LEVEL
   ld bc,0
InitLevelLoop:
   ld a,(de)
   bit 6,a                      ;if 2nd MSB set, it's an animated character
   push af
   call nz,&InitCharacter       ;if char, get hl for sprite from animations
   ld hl,&Empty                 ; and draw empty sprite
   pop af
   call z,&SpriteLookUp         ;else, get hl for sprite from sprite table
   call &AllignedSprite
   inc de
   ld a,8
   add a,b
   ld b,a
   cp 128
   jr c,InitLevelLoop
   ld b,0
   ld a,8
   add a,c
   ld c,a
   cp 56
   jr c,InitLevelLoop

   ld b,16
   ld hl,GRAPH_MEM+896
BottomLine:
   ld (hl),$FF
   inc hl
   djnz BottomLine
   ret


;input:
;  a has character number
;  b,c has the x,y position
;  de has address of sprite in map
;output:
;  none
;destroys:
;  af,hl
InitCharacter:
   ex de,hl
   ld (hl),EMPTY
   ex de,hl
   cp SOLOMON                   ;is the character solomon?
   jr z,CharIsSol               ;yes, it is sol, so jump

CharIsEnemy:                    ;else, it is an enemy
   push de
   ld hl,NUMENEMIES
   inc (hl)                     ;one more enemy
   ld (ix+CHARNUM),a
   ld (ix+DELAY),1
   ld (ix+XPOS),b
   ld (ix+YPOS),c
   ;some enemies use EXTRA field for it original location
   ;so put this in extra for all enemies
   call &GetMapped              ;de has mapped position
   ld a,d
   rrca \ rrca \ rrca \ rrca    ;put x pos in high nibble (low nibble is 0)
   or e                         ;y position in low nibble
   ld (ix+EXTRA),a              ;save the mapped position
   ld de,SIZEOFSTRUCT
   add ix,de                    ;point to next enemy struct
   pop de
   ret

CharIsSol:
   push ix                      ;save ix since used to point to next enemy
   ld ix,SOLSTRUCT
   ld (ix+CHARNUM),a
   ld a,1
   ld (ix+DELAY),a
   ld (ix+XPOS),b               ;save x,y position
   ld (ix+YPOS),c
   pop ix
   ret


;Input:
;  DE has the mapped (x,y) position of sprite
;Output:
;  HL has address of sprite at map (x,y)
;destroys:
;  none
GetHLSprite:
   push af
   push de
   ld a,e
   rlca                         ;\
   rlca                         ; |mult by 16 (16 is #columns in a row)
   rlca                         ; |
   rlca                         ;/ 
   add a,d                      ;a now contains the byte offset in map (a<112)
   ld hl,LEVEL                  ;hl has beginning of current level
   ld d,0                       ;\de now has byte offset in map
   ld e,a                       ;/
   add hl,de                    ;hl has address of tile 
   pop de
   pop af
   ret


;input:
;  a has the Nth byte (the offset) of the level
;output:
;  de has mapped x,y position
;destroys:
;  af
GetMappedFromOffset:
   ld e,a
   and %00001111                ;only look at mapped x
   ld d,a                       ;put mapped x in d
   sra e \ sra e                ;\put mapped y
   sra e \ sra e                ;/ in e
   ret


;Input:
;  DE has mapped (x,y) of sprite at (B,C)'s top left corner
;Output:
;  BC has (x,y) of top left corner of sprite
;Destroys:
;  none
GetPosition:
   ld b,d
   ld c,e
   sla b                        ;\
   sla b                        ; | mult by 8 (pixel / sprite)
   sla b                        ;/
   sla c                        ;\
   sla c                        ; | mult by 8
   sla c                        ;/
   ret



;Input:
;  BC has (x,y) of top left corner of sprite
;Output:
;  DE has mapped (x,y) of sprite at (B,C)'s top left corner
;Destroys:
;  none
GetMapped:
   ld d,b
   ld e,c
   sra d                        ;\
   sra d                        ; |div by 8 (map pixel size)
   sra d                        ;/
   sra e                        ;\
   sra e                        ; |div by 8
   sra e                        ;/
   ret



;input:
;  a has the sprite number
;output:
;  hl has the address of the sprite gfx
;destroys:
;  af
SpriteLookUp:
   bit 7,a                      ;is the sprite the block or wall?
   jr nz,BlockOrWall            ;yes, so branch
   push bc
   push de
   and %00111111                ;do not look two most significant bits
   ld l,a
   ld h,8
   call MUL_HL
   ld bc,&Sprites
   add hl,bc                    ;hl has the address of the sprite
   pop de
   pop bc
   ret
;the block sprites do not look at the 6 low order bits so objects can
;  hidden underneath them.  a level cannot start with a cracked block!
BlockOrWall:
   ld hl,&Wall                  ;assume its the wall sprite
   cp WALL                      ;is assumption correct?
   ret z                        ;yes, so return
   ld hl,&Block                 ;assume it is the uncracked block
   bit 6,a                      ;bit 6 set when cracked block
   ret z                        ;not set so return
   ld hl,&BlockLgCrk            ;else use cracked block block sprite
   ret


;destroys all regs
LoadLevel:
   call CLEARLCD

   ld hl,LEVELTIME              ;amount of time for each level
   ld (GAMETIMER),hl

   xor a
   ld (SOLSTRUCT+FLAGS),a       ;reset sol's flags

   ld (FBSTRUCT+EXTRA),a        ;reset fireball time to 0

   ld hl,GRAPH_MEM
   ld bc,1024-1
   call OTH_CLEAR               ;clear the graph mem
   ld hl,TEXT_MEM2
   ld bc,168-1
   call OTH_CLEAR               ;clear mem for enemy structs

   call &DrawScroll             ;draw the fireball scroll

   call &DecompLevel            ;decompress individual level
   ret


;Input:
;  hl has physical address to beginning of character's animation
;  ix points to character's information
;Output:
;  hl has address of sprite to be drawn
;  update sprite's offset, delay, and sprtaddr
;Destroys:
;  de
SpriteAnim:
   push af
   ld a,(ix+OFFSET)             ;ld sprite offset in a
   add a,9                      ;increment to next sprite
   ld (ix+OFFSET),a             ;save the increment
   ld d,0                       ;\de has offset
   ld e,a                       ;/
   add hl,de                    ;add to hl to point to next delay & sprite
   ld a,(hl)                    ;a has new delay
   or a                         ;if new delay is 0
   jr nz,SADelayNot0            ;then hl - next value
   inc hl
   ld e,(hl)                    ;\
   ld d,$FF                     ; |move back to beginning of animation
   add hl,de                    ;/
   ld a,(hl)
   ld (ix+OFFSET),0             ;sprite offset back to 0
SADelayNot0:
   ld (ix+DELAY),a              ;ld new delay into character's info
   inc hl                       ;(hl) has sprite to draw
   ld (ix+SPRTADDR),h           ;\ save the address of the sprite
   ld (ix+SPRTADDR+1),l         ;/
   pop af
   ret


;input:
;  bc has x,y (x position should be a multiple of 8)
;  hl has sprite
;destroys:
;  none
AllignedSprite:
   push hl
   push de
   push bc
   ld a,63                      ;\
   sub c                        ; | 63 - y position to move origin
   ld c,a                       ;/  to top left
   ex de,hl                     ;put location of sprite in de
   call FIND_PIXEL              ;hl has offset
   ld bc,GRAPH_MEM              ;draw directly to video memory
   add hl,bc                    ;location of where to start drawing
   ld b,8
ASLoop:
   push bc                      ;save count in b
   ld a,(de)                    ;\draw first 8 pixels of row
   ld (hl),a                    ;/
   inc de                       ;next row of sprite to draw
   ld bc,16                     ;\move to the next row of video memory
   add hl,bc                    ;/
   pop bc                       ;restore count in b
   djnz ASLoop
   pop bc
   pop de
   pop hl
   ret

            
PutSpriteOR:
      push ix
      push bc
      push de
      ld a,c
      cp -3
      jr c,PSOR_NotTooSmall
      ld c,0
PSOR_NotTooSmall:
      ld a,63              ;\
      sub c                ; |63-y, so the drawing starts at the top
      ld c,a               ;/
      push hl
      push hl
      call FIND_PIXEL      ;hl has offset in viedo mem; a has bit^2
      ld de,(VIRTUALSCREEN);draw to virtual screen
      add hl,de
      ex de,hl             ;de physical address in vid mem
      pop hl               ;hl has sprite's beginning address again
      ld bc,$0808          ;8 is width and height
      push hl              ;(hl) has the graphics of the sprite
      pop ix               ;(ix) has the graphics of the sprite
      ex de,hl             ;\(de) has gfx of sprite
                                                   ;/hl has addr. in video mem
PSOR_NewRow:
      push bc
      ld d,(ix)            ;d has graphics for one row of sprite
      inc ix               ;point to next row of sprite
      push af
      push hl
PSOR_NewCol:
      rl d
      ld e,a
      jr nc,PSOR_NextPixel
      or (hl)
      ld (hl),a
PSOR_NextPixel:
      ld a,e               ;restore bit^2 into a
      rrca                 ;\circular rotation of a, so a affects next bit
      jr nc,PSOR_SameByte  ;C flag set when the 1 in a is shifted out of a
      inc hl               ;next byte in video memory
PSOR_SameByte:
      djnz PSOR_NewCol     ;go to the next column if b not = 0
      pop hl               ;restore the video mem address
      pop af               ;restore the byte from FIND_PIXEL
      ld de,16             ;\move down a row (which is 16 bytes)
      add hl,de            ;/
      pop bc               ;bc again contains the width and height
      dec c                ;one less row to draw
      jr nz,PSOR_NewRow    ;when c=0, don't draw anymore rows
      pop hl               ;\
      pop de               ; |pixel plotted so restore the registers
      pop bc               ; |
      pop ix               ;/
      ret


;Square Sprite collision routine.
;Input:
;  a has the size of the length of the square edge (a = 8 for 8 pixel sprite)
;  bc has (x,y) of top left of first sprite
;  de has (x,y) of top left of second sprite
;Output:
;  carry flag set if no collision occurs
;  carry flag reset on collision
;Destroys:
;  a
SpriteCollision:
   dec a
   add a,b
   cp d
   ret c                        ;no collision
   sub b                        ;reg a again has edge length
   add a,d
   cp b
   ret c                        ;no collision
   sub d
   add a,c
   cp e
   ret c                        ;no collision
   sub c
   add a,e
   cp c
   ret c                        ;no collision
   or a                         ;clear the carry, collision!
   ret


;input:
;  none
;output:
;  hl has address of first compressed level
;destroys:
;  af,bc,de
DispLevelScreen:
   call CLEARLCD
   ld ($800C),bc                ;bc = 0 from CLEARLCD
   set 3,(iy+5)                 ;reverse video writing
   ld hl,&SolsKeyText
   call D_ZT_STR                ;display sol's key title
   res 3,(iy+5)
   ld hl,(WORLDTEXTADDR)
   ld a,21                      ;maximum length of world name string
   sub (hl)                     ;subtract length of string
   rra                          ;divide by 2 to get center (sub resets c flag)
   ld b,a
   ld c,2
   ld ($800C),bc
   call D_LT_STR                ;hl has length of author's name
   push hl
   ld de,$0904
   ld ($800C),de
   ld hl,LEVELNUM
   ld l,(hl)
   inc l
   ld h,0
   call D_HL_DECI
   ld de,$0604
   ld ($800C),de
   ld hl,&LevelNumText
   call D_ZT_STR
   ld de,$3869
   ld ($8333),de
   ld hl,SOLLIVES
   ld l,(hl)
   ld h,0
   call DM_HL_DECI
   ld de,$385C
   ld ($8333),de
   ld hl,&LivesCountText
   call D_ZM_STR
   pop hl                       ;hl has author's name
   ld bc,$3802
   ld ($8333),bc
   ld b,(hl)
   inc hl
   call D_LM_STR                ;hl has addr of number of levels in string
   ld a,(hl)
   ld (NUMLEVELS),a             ;next byte is number of levels
   inc hl                       ;point to 1st level's data
   call OTH_PAUSE               ;wait for user to be ready
   push hl
   call &Transition
   pop hl
   ret

LevelNumText:
.db "Level",0
LivesCountText:
.db "Lives",0


;input:
;  none
;output:
;  c set if no levels in VAT
;  c reset if level selected
;    hl points to address of name of level selected (3rd byte of string)
;destroys:
;  all
ChooseWorld:
   ld b,0                       ;b = world #
CWCountWorldsLoop:
   push bc
   call &FindVATEntry           ;does world #b exist??
   pop bc
   inc b                        ;# of next world
   jr nc,CWCountWorldsLoop      ;search for next world
CWHaveLastWorld:
   dec b                        ;total 1 too high (inc'ed when none found)
   jr z,CWNoWorlds              ;are there 0 levels in VAT????

   ld a,b
   ld (NUMWORLDS),a             ;save the number of worlds

   ;now display the worlds screen
   xor a                        ;a is the world at top of page
   call &DrawWorldsScreen       ;draw page 1 of worlds
                                ; b=0 - initial world pointer
                                ; c=0 - initial top world on page pointer
CWTopOfLoop:
   call GET_KEY
   halt                         ;get rid of some flickering of video
   cp K_LEFT
   jr z,CWLeft                  ;go to previous page
   cp K_RIGHT
   jr z,CWRight                 ;go to next page
   cp K_UP
   jr z,CWUp                    ;move cursor up
   cp K_DOWN
   jr z,CWDown                  ;move cursor down
   cp K_ENTER
   jr z,CWOutOfLoop             ;select this level!!!
   cp K_EXIT
   jr z,CWReturnNoLevel
   jr CWTopOfLoop               ;nothing intersting pressed so try again

CWLeft:
   ld a,c
   or a                         ;are we at first page?
   jr z,CWTopOfLoop             ;yep, so no previous page
   sub 8                        ;else move to previous page
   call &DrawWorldsScreen       ;b,c = top world of new screen
   jr CWTopOfLoop

CWRight:
   ld a,(NUMWORLDS)
   sub c
   cp 9                         ;are we on last page?
   jr c,CWTopOfLoop             ;yes, so no next page to go to
   ld a,c
   add a,8                      ;move to next page
   call &DrawWorldsScreen       ;b,c = top world of new screen
   jr CWTopOfLoop

CWUp:
   ld a,b
   and 7                        ;only look at 0-7
   or a                         ;on top row?
   jr z,CWTopOfLoop             ;yep, so no world above
   call &InvertRow              ;else reset this row to normal
   dec b
   call &InvertRow              ; and invert this row
   jr CWTopOfLoop

CWDown:
   ld a,(NUMWORLDS)
   sub 2
   jr c,CWTopOfLoop             ;if only 1 world, don't move down
   cp b                         ;trying to move passed last level?
   jr c,CWTopOfLoop             ;yes so nothing to do
   ld a,b
   and 7                        ;only look at 0-7
   cp 7                         ;on bottom row?
   jr nc,CWTopOfLoop            ;yep, so no world below
   call &InvertRow              ;else reset this row to normal
   inc b
   call &InvertRow              ; and invert this row
   jr CWTopOfLoop

CWOutOfLoop:
   call &FindVATEntry           ;get pointer to level and reset c flag
   ret

CWNoWorlds:
   ld a,1
   ld (NUMWORLDS),a
   dec a
   call &DrawWorldsScreen
   ld bc,0
   call &InvertRow
   ld hl,&NoWorldsText
   call &DispMenuEntry
   call OTH_PAUSE
CWReturnNoLevel:
   scf                          ;set carry flag since no level selected
   ret


LevelMenuText:
.db "Level Select      X/X",0   ;the X's get replaced

NoWorldsText:
.db 16,"No levels found!"




;input:
;  a - world at top row of menu (must be valid and divisible by 8)
;  NUMWORLDS
;output:
;  b,c = input a
;destroys:
;  af,de,hl
DrawWorldsScreen:
   ld c,a
   ld b,a
   push bc
   ld hl,&LevelMenuText+18      ;character for current page
   rrca \ rrca \ rrca           ;divide world # at top row to get page #
   add a,49                     ;convert to ascii (+1)
   ld (hl),a                    ;store in string
   inc hl \ inc hl              ;character for number of total pages
   ld a,(NUMWORLDS)
   dec a
   and %11111000                ;clear all the bits to rotate (divide)
   rrca \ rrca \ rrca           ;divide by 8
   add a,49                     ;convert to ascii (+1)
   ld (hl),a                    ;store in string
   push bc
   call CLEARLCD                ;de = 0 from CLEARLCD
   pop bc
   ld ($800C),de
   set 3,(iy+5)                 ;reverse video writing
   ld hl,&LevelMenuText
   call D_ZT_STR
   res 3,(iy+5)
   ld b,c                       ;look in VAT for top row initially
   ld c,0                       ;row number to write at and count
DWSLoop:
   push bc
   call &FindVATEntry           ;hl points to level name if level exists
   pop bc
   jr c,DWSNoMoreEntries        ;no more levels in VAT
   push bc
   call &DispMenuEntry
   pop bc
   inc b
   inc c
   ld a,c
   cp 8
   jr c,DWSLoop                 ;loop until 8 names written
DWSNoMoreEntries:
   pop bc
   call &InvertRow              ;invert top row
   ret



;input:
;  c - row number to display menu text (0-7)
;  hl - address of text (must be length indexed)
;destroys:
;  af,bc,hl
DispMenuEntry:
   ld b,c
   ld a,2                       ;initial y pos for row 0 (-6, so init Y is 8)
   inc b
DMELoop:
   add a,6                      ;6 rows of pixel / row of text
   djnz DMELoop
   ld b,a                       ;y position of text
   ld c,3                       ;x position of text
   ld ($8333),bc
   ld b,(hl)                    ;index in b
   inc hl                       ;text addr in hl
   call D_LM_STR
   ret


;input:
;  b - row of video to invert (0 - 7) (only looks at 3 LSB's)
;destroys:
;  a,hl,de
InvertRow:
   ld hl,VIDEO_MEM+48
   ld de,96                     ;# bytes for 6 rows of pixels across screen
   ld a,b
   and %00000111                ;only look at low 3 bits (0-7)
   inc a
IR_Loop:
   add hl,de
   dec a
   jr nz,IR_Loop                ;hl has addr if video memory to invert
   ld e,96
IR_InvertLoop:
   ld a,(hl)
   cpl
   ld (hl),a
   inc hl
   dec e
   jr nz,IR_InvertLoop
   ret



;input:
;  b - the Nth entry sol's key v1.0 level ( 0,1,2,..,N-1 if N levels )
;output:
;  c flag set if entry does not exist
;  c flag reset
;    hl - pointer to Level name (length indexed, not null terminated)
;destroys:
;  af,bc,de,hl
FindVATEntry:
   inc b                        ;start at 1 instead of 0
   ld hl,VAT_START-10           ;scan VAT, skip first entry
FVE_Loop:
   ld a,$0C                     ;search for strings
   push bc
   call SEARCH_VAT              ;de now has first byte of data for string
   pop bc
   ret c                        ;finished searching VAT
   ld a,(de)
   cp USGARDID                  ;is it a sol's key lvl? (1st byte of string)
   inc de
   ld a,(de)
   cp 'S'                       ;check for 'S' char used to make sure it's mine
   jr nz,FVE_Loop               ;string was not a sol level so look for next
   inc de
   ld a,(de)
   cp 'K'                       ;check for 'K' char used to make sure it's mine
   jr nz,FVE_Loop               ;string was not a sol level so look for next
   inc de
   ld a,(de)                    ;put level version in a
   ld c,a                       ;save level version in c
   and %01111111                ;high bit used for enable/disable cheats
   or a                         ;is level version 0 (1st version)?
   jr nz,FVE_Loop               ;version not supported sol look for next

   ;if you get here, string is a Sol's Key level v1.0
   djnz FVE_Loop                ;find the Nth sol's key level, dec b

   ;if you get here, string is level N
   ld hl,GAMEFLAGS
   set CHEATENABLED,(hl)        ;assume ok to cheat
   rl c                         ;put cheat bit in carry flag
   jr c,FVE_CheckLevelVersion   ;ok to cheat
   res CHEATENABLED,(hl)        ;can't cheat
FVE_CheckLevelVersion:
   inc de                       ;after version number comes level's name
   ex de,hl                     ;put pointer to level name in hl
   or a                         ;reset carry flag
   ret



;input:
;  LEVELPTR has addr of level to decompress (first byte is # of bytes in level)
;output:
;  NEXTLEVELPTR has address of next level
;  LEVEL has the data for the next level
;destroys:
;  all
DecompLevel:
   ld hl,(LEVELPTR)             ;points to level (1st byte is # of bytes in level)
   inc hl                       ;point to 1st most freq tile
   ld (TEMPWORD),hl             ;store the addr of most freq tiles in TEMPWORD
   inc hl                       ;point to 2nd most freq tile
   inc hl                       ;point to 3rd most freq tile
   inc hl                       ;hl has addr of bitmap
   push hl
   ld de,28                     ;bitmap is 28 bites (2 bits for each tile)
   add hl,de                    ;hl has addr of infreq tile list
   pop de                       ;de has addr of bitmap
   ex de,hl                     ;SWAP!
   ld c,$80                     ;bit mask for 2 bits starting at MSBs
   ld b,112                     ;# of bytes in a level
   ld ix,LEVEL
DL_Loop:
   ld a,c                       ;put mask in a
   and (hl)                     ;is this bit set?
   push de
   ld de,(TEMPWORD)
   jr z,DL_0or1                 ;bit isn't set, so bitmap is 0 or 1
DL_2or3:
   inc de                       ;address of second most frequent tile
   rrc c                        ;\
   ld a,c                       ; | see what the next bit following the 1 is
   and (hl)                     ;/
   ld a,(de)                    ;we will assume it is a 0 (10b - 2nd most freq)
   jr z,DL_Got2or3              ;if assumption is correct, jump
   inc de                       ;\ lets get the 3rd most frequent tile
   ld a,(de)                    ;/    (11b - 3rd most frequent tile)
DL_Got2or3:
   pop de
   jr DL_PutTileInLevel         ;ok, we now know what tile to put in the level
DL_0or1:
   rrc c                        ;\ 
   ld a,c                       ; | see what the next bit following the 0 is
   and (hl)                     ;/
   ld a,(de)                    ;we will assume it is a 1 (01b - 1st most freq)
   pop de
   jr nz,DL_Got0or1             ;if assumption correct, jump
   ld a,(de)                    ;get the tile from the infrequent tiles list (00b)
   inc de                       ;move to next infrequent tile
DL_Got0or1:
DL_PutTileInLevel:
   ld (ix),a                    ;put the tile in the level
   inc ix                       ;next write will be at next byte
   rrc c                        ;move to 1st of next 2 bits of bitmap
   jr nc,DL_SameBitmapByte      ;did this cause use to go to the next byte of bmp
   inc hl                       ;yes, so move to next byte of bitmap
DL_SameBitmapByte:
   djnz DL_Loop                 ;read in 112 bytes yet?
   ld (NEXTLEVELPTR),de         ;save address of next level
   ret




TimerInterrupt:
   ld hl,INTTIMER
   inc (hl)
   ret


JumpingArc:
.db 1,2,2,2,1,1,0,-1,$F0        ;# pixels to move y pos, $f0 terminates



#include "solskey.gfx"
#include "solskey.ani"


.end
