import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

public class ImageButton extends java.awt.Canvas
{
    static Image upButton = null;
    static Image downButton = null;
    Image gifImage;
    boolean isPressed = false;
    int buttonWidth;
    int buttonHeight;
    int imageWidth;
    int imageHeight;
    ActionListener actionListener = null;
    MouseListener mouseListener = null;
    Object ib = this;
    
    ImageButton(Image image)
    {
        this( image, 40, 40, 32, 32 );
    }
    
    ImageButton(Image image, int bWidth, int bHeight, int iWidth, int iHeight)
    {
        gifImage = image;
        buttonWidth = bWidth;
        buttonHeight = bHeight;
        imageWidth = iWidth;
        imageHeight = iHeight;
        
        setSize(bWidth, bHeight);
    }
    
    public Dimension getPreferredSize()
    {
        return new Dimension(buttonWidth, buttonHeight);
    }
    
    public Dimension getMinimumSize()
    {
        return new Dimension(buttonWidth, buttonHeight);
    }
    
    public Dimension getMaximumSize()
    {
        return new Dimension(buttonWidth, buttonHeight);
    }
    
    public boolean isPressed()
    {
        return isPressed;
    }
    
    public void setPressed(boolean isPressed)
    {
        this.isPressed = isPressed;
        repaint();
    }
    
    public void addActionListener(ActionListener l)
    {
        // button listens for a mouse click
        mouseListener = new ImageButtonMouseListener( this );
        addMouseListener( mouseListener );
        
        // generates an action event (like normal Button's do)
        actionListener = AWTEventMulticaster.add(actionListener, l);
    }

    public void removeActionListener(ActionListener l)
    {
        // don't listen for mouse clicks anymore
        removeMouseListener( mouseListener );
        mouseListener = null;
        
        // no more action events
        actionListener = AWTEventMulticaster.remove(actionListener, l);
    }

    private class ImageButtonMouseListener extends MouseAdapter
    {
        Object host;
        
        public ImageButtonMouseListener(Object host)
        {
            this.host = host;
        }
        
        public void mousePressed(MouseEvent e)
        {
            if ( actionListener != null )
            {
                isPressed = (isPressed) ? false : true;
                repaint();

                // create a new ActionEvent like a java.awt.Button does
                actionListener.actionPerformed( new ActionEvent(host, ActionEvent.ACTION_PERFORMED, "ImageButton pressed") );
            }
        }
    }

    public void paint(Graphics g)
    {
        // create the empty upButton image
        if (upButton == null)
            createUpButton();
        if (downButton == null)
            createDownButton();

        Image imageBuf = createImage( buttonWidth, buttonHeight );
        Graphics gBuf = imageBuf.getGraphics();

        if ( isPressed )
        {
            gBuf.drawImage(downButton,0,0,this);
            gBuf.drawImage(gifImage,4,4,imageWidth,imageHeight,this);
        }
        else
        {
            gBuf.drawImage(upButton,0,0,this);
            gBuf.drawImage(gifImage,3,3,imageWidth,imageHeight,this);
        }
        g.drawImage(imageBuf,0,0,this);

    }

    private void createUpButton()
    {
        // button background is this color
        setBackground(new Color(0xC0,0xC0,0xC0));

        // button size
        upButton = createImage(buttonWidth,buttonHeight);

        // get graphics to paint on
        Graphics g = upButton.getGraphics();

        g.setColor(new Color(0xE0,0xE0,0xE0));
        g.drawLine(0,0,0,buttonWidth-2);
        g.drawLine(0,0,buttonHeight-2,0);
        g.setColor(new Color(0x00,0x00,0x00));
        g.drawLine(buttonHeight-1,0,buttonHeight-1,buttonWidth-1);
        g.drawLine(0,buttonWidth-1,buttonHeight-1,buttonWidth-1);
        g.setColor(new Color(0x80,0x80,0x80));
        g.drawLine(buttonHeight-2,1,buttonHeight-2,buttonWidth-2);
        g.drawLine(buttonHeight-3,2,buttonHeight-3,buttonWidth-3);
        g.drawLine(1,buttonWidth-2,buttonHeight-2,buttonWidth-2);
        g.drawLine(2,buttonWidth-3,buttonHeight-3,buttonWidth-3);
    }
    
    private void createDownButton()
    {
        // button background is this color
        setBackground(new Color(0xC0,0xC0,0xC0));

        // button size
        downButton = createImage(buttonWidth,buttonHeight);

        // get graphics to paint on
        Graphics g = downButton.getGraphics();

        g.setColor(new Color(0x00,0x00,0x00));
        g.drawLine(0,0,buttonHeight-2,0);
        g.drawLine(0,0,0,buttonWidth-2);
        g.setColor(new Color(0xD0,0xD0,0xD0));
        g.drawLine(buttonHeight-1,0,buttonHeight-1,buttonWidth-1);
        g.drawLine(0,buttonWidth-1,buttonHeight-1,buttonWidth-1);
        g.setColor(new Color(0x80,0x80,0x80));
        g.drawLine(1,1,buttonHeight-3,1);
        g.drawLine(2,2,buttonHeight-4,2);
        g.drawLine(1,1,1,buttonWidth-3);
        g.drawLine(2,2,2,buttonWidth-4);
        g.setColor(new Color(0xB0,0xB0,0xB0));
        g.drawLine(buttonHeight-2,1,buttonHeight-2,buttonWidth-2);
        g.drawLine(buttonHeight-3,2,buttonHeight-3,buttonWidth-3);
        g.drawLine(1,buttonWidth-2,buttonHeight-2,buttonWidth-2);
        g.drawLine(2,buttonWidth-3,buttonHeight-3,buttonWidth-3);
    }

}