
import java.awt.*;
import java.awt.event.*;

public class SolLevel extends Panel
{
    TileCanvas[][] tileCanvases = null;
    SolEditor app = null;
    LevelMouseListener l = new LevelMouseListener();

    SolLevel(SolEditor app)
    {
        this( null, app );
    }

    SolLevel(SolTile[][] levelTiles, SolEditor app)
    {
        this.app = app;

        if ( levelTiles == null )
        {
            // create an empty level
            levelTiles = new SolTile[SolEditor.LEVELROWS][SolEditor.LEVELCOLS];
            for (int i=0; i<SolEditor.LEVELROWS; i++)
                for (int j=0; j<SolEditor.LEVELCOLS; j++)
                    levelTiles[i][j] = SolEditor.solTiles[0];
        }
        else
        {
            // already have data for level
            levelTiles = levelTiles;
        }

        setSize( new Dimension( SolEditor.IMAGEWIDTH * SolEditor.LEVELCOLS,
                                SolEditor.IMAGEHEIGHT * SolEditor.LEVELROWS ) );

        setLayout( new GridLayout(SolEditor.LEVELROWS, SolEditor.LEVELCOLS) );
        ((GridLayout)getLayout()).setHgap(0);
        ((GridLayout)getLayout()).setVgap(0);
        
        tileCanvases = new TileCanvas[SolEditor.LEVELROWS][SolEditor.LEVELCOLS];
        for (int i=0; i<SolEditor.LEVELROWS; i++)
        {
            for (int j=0; j<SolEditor.LEVELCOLS; j++)
            {
                tileCanvases[i][j] = new TileCanvas( levelTiles[i][j] );

                tileCanvases[i][j].addMouseListener( l );

                add( tileCanvases[i][j] );
            }
        }
    }

    public Dimension getPreferredSize()
    {
        return new Dimension( SolEditor.IMAGEWIDTH * SolEditor.LEVELCOLS,
                              SolEditor.IMAGEHEIGHT * SolEditor.LEVELROWS );
    }
    
    public Dimension getMinimumSize()
    {
        return new Dimension( SolEditor.IMAGEWIDTH * SolEditor.LEVELCOLS,
                              SolEditor.IMAGEHEIGHT * SolEditor.LEVELROWS );
    }
    
    public Dimension getMaximumSize()
    {
        return new Dimension( SolEditor.IMAGEWIDTH * SolEditor.LEVELCOLS,
                              SolEditor.IMAGEHEIGHT * SolEditor.LEVELROWS );
    }
    
    class LevelMouseListener extends MouseAdapter
    {
        public void mousePressed(MouseEvent e)
        {
            TileCanvas tileCanvas = (TileCanvas)e.getSource();

            // if *only* normal mouse button pressed, select next tile
            if ( e.getModifiers() == MouseEvent.BUTTON1_MASK )
            {
                // it is not this type of tile so show first tile of this type
                tileCanvas.setTile( (app.currentTile.isSameTileType(tileCanvas.getTile())) ? 
                    tileCanvas.getTile().getNext() : 
                    app.currentTile );
            }
            // else some other mouse event (shift+click, right-click, etc), so select 
            //  toggle between tile on button and an empty
            else
            {
                if ( app.currentTile.isSameTileType( tileCanvas.getTile() ) )
                    tileCanvas.setTile( app.empty1 );
                else
                    tileCanvas.setTile( app.currentTile );
            }

            // display the new tile
            tileCanvas.repaint();
        }
    }

    public SolTile[][] getLevel()
    {
        SolTile[][] level = new SolTile[tileCanvases.length][tileCanvases[0].length];
        
        for (int i=0; i<level.length; i++)
        {
            for (int j=0; j<level[i].length; j++)
            {
                level[i][j] = tileCanvases[i][j].getTile();
            }
        }
            
        return level;
    }

    public byte[] getLevelBytes()
    {
        byte[] a = new byte[tileCanvases.length * tileCanvases[0].length];

        for (int i=0; i<tileCanvases.length; i++)
        {
            for (int j=0; j<tileCanvases[0].length; j++)
            {
                a[i*tileCanvases[0].length + j] = tileCanvases[i][j].getTile().getValue();
            }
        }

        return a;
    }

    public void repaint()
    {
        super.repaint();
        
        for (int i=0; i<tileCanvases.length; i++)
        {
            for (int j=0; j<tileCanvases[i].length; j++)
            {
                tileCanvases[i][j].repaint();
            }
        }
    }
}
